from .on_message import OnMessage
from .on_callback_query import OnCallbackQuery
from .on_channel_created import OnChannelCreated
from .on_channel_deleted import OnChannelDeleted
from .on_channel_updated import OnChannelUpdated
from .on_command import OnCommand
from .on_community_updated import OnCommunityUpdated
from .on_group_created import OnGroupCreated
from .on_group_deleted import OnGroupDeleted
from .on_group_updated import OnGroupUpdated
from .on_member_joined import OnMemberJoined
from .on_member_left import OnMemberLeft
from .on_user_banned import OnUserBanned
from .on_unknown_command import OnUnknownCommand
from .on_inline_query import OnInlineQuery


class Decorators(
    OnMessage,
    OnCallbackQuery,
    OnChannelCreated,
    OnChannelDeleted,
    OnChannelUpdated,
    OnCommand,
    OnCommunityUpdated,
    OnGroupCreated,
    OnGroupDeleted,
    OnGroupUpdated,
    OnMemberJoined,
    OnMemberLeft,
    OnUserBanned,
    OnUnknownCommand,
    OnInlineQuery,
):
    pass



from typing import Type, TypeVar
import swibots
from swibots.api.chat.models import Message


class DeleteMessage:
    async def delete_message(self: "swibots.ApiClient", message: int | Message) -> bool:
        """Delete a message

        Parameters:
            message (``int`` | ``~switch.api.chat.models.Message``): The message to delete

        Returns:
            ``bool``: Whether the message was deleted

        Raises:
            ``~switch.error.SwitchError``: If the message could not be deleted

        This method does the same as :meth:`~switch.api.chat.controllers.MessageController.delete_message`.
        """
        return await self.chat_service.messages.delete_message(message)



from swibots.utils.ws.asyncstomp import AsyncWsClient


class SwitchWSAsyncClient(AsyncWsClient):
    def __init__(self, url: str, token: str = None):
        super().__init__(url)
        self._token = token

    @property
    def token(self) -> str:
        return self._token

    @token.setter
    def token(self, value: str):
        self._token = value

    def _set_default_headers(self, headers):
        headers = super()._set_default_headers(headers)
        headers["Authorization"] = f"Bearer {self._token}"
        return headers



from typing import Optional
import swibots
from .community_event import CommunityEvent
from swibots.api.community.models.channel import Channel
from swibots.api.community.models.community import Community
from swibots.api.community.models.group import Group
from swibots.api.common.models.user import User
from swibots.types import EventType


class ChannelUpdatedEvent(CommunityEvent["ChannelUpdatedEvent"]):
    def __init__(
        self,
        app: "swibots.App" = None,
        community_id: Optional[str] = None,
        community: Optional[Community] = None,
        group_id: Optional[str] = None,
        group: Optional[Group] = None,
        channel_id: Optional[str] = None,
        channel: Optional[Channel] = None,
        action_by_id: Optional[str] = None,
        action_by: Optional[User] = None,
        data: Optional[dict] = None,
        user_id: Optional[str] = None,
        user: Optional[User] = None,
    ):
        super().__init__(
            app=app,
            type=EventType.CHANNEL_UPDATE,
            data=data,
            action_by=action_by,
            action_by_id=action_by_id,
            community=community,
            community_id=community_id,
            group=group,
            group_id=group_id,
            channel=channel,
            channel_id=channel_id,
            user=user,
            user_id=user_id,
        )



# import the standard JSON parser
import json
import logging
from typing import Optional, Tuple

# import the REST library
import httpx

from swibots.error import NetworkError, SwitchError
from swibots.utils.types import JSONDict

log = logging.getLogger(__name__)

DEFAULT_HEADERS = {"Accept": "application/json"}


class RestClient:
    def __init__(
        self,
        connection_pool_size: int = 1,
        proxy_url: str = None,
        read_timeout: Optional[float] = None,
        write_timeout: Optional[float] = None,
        connect_timeout: Optional[float] = None,
        pool_timeout: Optional[float] = 1.0,
    ):
        timeout = httpx.Timeout(
            connect=connect_timeout,
            read=read_timeout,
            write=write_timeout,
            pool=pool_timeout,
        )
        limits = httpx.Limits(
            max_connections=connection_pool_size,
            max_keepalive_connections=connection_pool_size,
        )
        self._client_kwargs = dict(
            timeout=timeout,
            proxies=proxy_url,
            limits=limits,
        )

        try:
            self._client = self._build_client()
        except ImportError as exc:
            if "httpx[socks]" not in str(exc):
                raise exc

            raise RuntimeError(
                "To use Socks5 proxies, PTB must be installed via `pip install "
                "python-telegram-bot[socks]`."
            ) from exc

    def _build_client(self) -> httpx.AsyncClient:
        # type: ignore[arg-type]
        return httpx.AsyncClient(**self._client_kwargs)

    def initialize(self) -> None:
        log.debug("Initializing HTTPXRequest")
        if self._client.is_closed:
            self._client = self._build_client()

    async def shutdown(self) -> None:
        """See :meth:`BaseRequest.shutdown`."""
        if self._client.is_closed:
            log.debug("This HTTPXRequest is already shut down. Returning.")
            return
        await self._client.aclose()

    async def get(self, url: str, data: dict = None, headers: dict = None) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.get`."""
        return await self.do_request(url, "GET", data, headers=headers)

    async def post(self, url: str, data: dict = None, form_data=None, files=None, headers: dict = None) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.post`."""
        return await self.do_request(url, "POST", data, form_data, files, headers)

    async def put(self, url: str, data: dict = None, form_data=None, files=None, headers: dict = None) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.put`."""
        return await self.do_request(url, "PUT", data, form_data, files, headers)

    async def delete(self, url: str, data: dict = None, headers: dict = None) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.delete`."""
        return await self.do_request(url, "DELETE", data, headers=headers)

    async def patch(self, url: str, data: dict = None, form_data=None, files=None, headers: dict = None) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.patch`."""
        return await self.do_request(url, "PATCH", data, form_data, files, headers)

    async def head(self, url: str, data: dict = None, headers: dict = None) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.head`."""
        return await self.do_request(url, "HEAD", data, headers)

    async def options(
        self, url: str, data: dict = None, headers: dict = None
    ) -> Tuple[int, bytes]:
        """See :meth:`BaseRequest.options`."""
        return await self.do_request(url, "OPTIONS", data, headers)

    def prepare_request_data(self, data: dict) -> str | None:
        data = {**data} if data else None
        return data

    def prepare_request_headers(self, headers: dict) -> dict:
        if headers is None:
            headers = {}
        reqHeaders = {**headers}
        return reqHeaders

    async def do_request(
        self, url: str, method: str, data: dict = None, form_data=None, files=None, headers: dict = None
    ) -> Tuple[int, bytes]:
        if self._client.is_closed:
            raise RuntimeError("This RestClient is not initialized!")
        try:
            data = self.prepare_request_data(data)
            reqHeaders = self.prepare_request_headers(headers)
            if (form_data is not None):
                # reqHeaders["Content-Type"] = "multipart/form-data"
                # reqHeaders["Accept"] = "application/json"
                response = await self._client.request(method, url, data=form_data, files=files, headers=reqHeaders)
            else:
                response = await self._client.request(method, url, json=data, headers=reqHeaders)
        except httpx.HTTPError as err:
            # HTTPError must come last as its the base httpx exception class
            # TODO p4: do something smart here; for now just raise NetworkError
            raise NetworkError(f"httpx HTTPError: {err}") from err

        return response.status_code, response.content

    @staticmethod
    def parse_json_payload(payload: bytes) -> JSONDict:
        """Parse the JSON returned from Switch.
        Tip:
            By default, this method uses the standard library's :func:`json.loads` and
            ``errors="replace"`` in :meth:`bytes.decode`.
            You can override it to customize either of these behaviors.
        Args:
            payload (:obj:`bytes`): The UTF-8 encoded JSON payload as returned by Telegram.
        Returns:
            dict: A JSON parsed as Python dict with results.
        Raises:
            SwitchError: If loading the JSON data failed
        """
        decoded_s = payload.decode("utf-8", "replace")
        try:
            return json.loads(decoded_s)
        except ValueError as exc:
            raise SwitchError("Invalid server response") from exc



import json
from typing import List, Tuple, Type, TypeVar
import swibots
from swibots.error import NetworkError
from swibots.utils import RestClient
from swibots.utils.types import JSONDict
from swibots.base import RestResponse

T = TypeVar("T", bound="swibots.SwitchObject")


class SwitchRestClient(RestClient):
    def __init__(self, app: "swibots.App" = None, base_url: str = None, token: str = None):
        super().__init__()
        self._app = app
        self._auth_token = token
        self._base_url = base_url

    @property
    def app(self) -> "swibots.App":
        return self._app

    @app.setter
    def app(self, app: "swibots.App"):
        self._app = app

    @property
    def token(self):
        return self._app.token

    @property
    def user(self) -> "swibots.AuthUser":
        return self._app.user

    @property
    def base_url(self, url: str):
        self._base_url = url

    @base_url.setter
    def base_url(self, url: str):
        self._base_url = url

    def build_object(self, obj_type: Type[T],  data: JSONDict) -> T:
        return obj_type.build_from_json(data, self.app)

    def build_list(self, obj_type: Type[T], data: JSONDict) -> List[T]:
        return obj_type.build_from_json_list(data, self.app)

    async def get(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.get`."""
        return await self.do_request(url, "GET", data, headers=headers)

    async def post(
        self, url: str, data: dict = None, form_data=None, files=None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.post`."""
        return await self.do_request(url, "POST", data, form_data, files, headers)

    async def put(
        self, url: str, data: dict = None, form_data=None, files=None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.put`."""
        return await self.do_request(url, "PUT", data, form_data, files, headers)

    async def delete(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.delete`."""
        return await self.do_request(url, "DELETE", data, headers=headers)

    async def patch(
        self, url: str, data: dict = None, form_data=None, files=None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.patch`."""
        return await self.do_request(url, "PATCH", data, form_data, files, headers)

    async def head(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.head`."""
        return await self.do_request(url, "HEAD", data, headers)

    async def options(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.options`."""
        return await self.do_request(url, "OPTIONS", data, headers)

    def parse_response(self, response: Tuple[int, bytes]) -> RestResponse[JSONDict]:
        decoded_s = response[1].decode("utf-8", "replace")
        try:
            jsonObject = json.loads(decoded_s)
        except ValueError as exc:
            jsonObject = decoded_s

        response = RestResponse(jsonObject, response[0], {})
        if response.is_error:
            raise NetworkError(response.error_message)
        return response

    def prepare_request_data(self, data: dict) -> dict:
        return super().prepare_request_data(data)

    def prepare_request_headers(self, headers: dict) -> dict:
        headers = super().prepare_request_headers(headers)
        if self.token is not None:
            headers["Authorization"] = f"Bearer {self.token}"
        return headers

    async def do_request(
        self, path: str, method: str, data: dict = None, form_data=None, files=None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        data = self.prepare_request_data(data)
        headers = self.prepare_request_headers(headers)
        return self.parse_response(
            await RestClient.do_request(self, self._base_url + (path if path is not None else ""), method, data, form_data, files, headers)
        )



from .utils import *
from .api import *
from .base import *
from .error import *
from .config import *
from .types import *

# from .bot_client import SwitchBotClient
from .app import App
from .bot_app import BotApp
from .bots import *



from typing import Type, TypeVar
import swibots
from swibots.api.auth.models import AuthUser

T = TypeVar("T", bound="swibots.AuthUser")


class Login:
    async def login(self: "swibots.ApiClient", username: str, password: str, user_type: Type[T] = AuthUser) -> T:
        """Login with username and password

        Parameters:
            username (``str``): The username   
            password (``str``): The password
            user_type (``Type[T]``, *optional*): The user type to return. Defaults to :obj:`~switch.api.auth.models.AuthUser`.

        Returns:
            ``T``: The user

        This functions does the same as :meth:`~switch.api.auth.controllers.UserController.login`.
        
        
        """
        return await self.auth_service.users.login(username=username, password=password, user_type=user_type)



from swibots.api.auth.controllers import UserController
from swibots.base import SwitchRestClient
from swibots.config import get_config
import swibots


class AuthClient(SwitchRestClient):
    """Auth client

    This client is used to communicate with the auth service.

    Controllers:
        - :attr:`users`: :obj:`~switch.api.auth.controllers.UserController` : The users controller
    """

    def __init__(self, app: "swibots.App", base_url: str = None):
        """Initialize the auth client"""
        base_url = base_url or get_config()["AUTH_SERVICE"]["BASE_URL"]
        super().__init__(app, base_url)
        self._users = None
        self._authorization = None

    @property
    def users(self) -> UserController:
        """Get the users controller

        Returns:
            :obj:`~switch.api.auth.controllers.UserController`: The users controller
        """
        if self._users is None:
            self._users = UserController(self)
        return self._users

    def prepare_request_headers(self, headers: dict) -> dict:
        headers = super().prepare_request_headers(headers)
        if self.token is not None:
            headers["authtoken"] = f"{self.token}"
        return headers



from .types import InlineQueryResultType
import swibots
from swibots.utils.types import JSONDict
from swibots.base import SwitchObject
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup
from .inline_query_result import InlineQueryResult


class BaseTypedInlineQueryResult(InlineQueryResult):
    def __init__(self,
                 id: str,
                 type: InlineQueryResultType,
                 title: str = None,
                 description: str = None,
                 thumb_url: str = None,
                 thumb_width: int = None,
                 thumb_height: int = None,
                 input_message: "InputMessageContent" = None,
                 reply_markup: "InlineMarkup" = None,
                 ):
        super().__init__(id, type, input_message, reply_markup)
        self.type = type
        self.title = title
        self.description = description
        self.thumb_url = thumb_url
        self.thumb_width = thumb_width
        self.thumb_height = thumb_height

    def to_json(self) -> JSONDict:
        data = super().to_json()
        data.update({
            "title": self.title,
            "description": self.description,
            "thumbUrl": self.thumb_url,
            "thumbWidth": self.thumb_width,
            "thumbHeight": self.thumb_height,
        })
        return data



import enum


class EventType(enum.Enum):
    """Represents the type of a event."""

    # Chat
    MESSAGE = "MESSAGE"
    COMMAND = "COMMAND"
    CALLBACK_QUERY = "CALLBACK_QUERY"
    INLINE_QUERY = "INLINE_QUERY"
    # Community
    CHANNEL_CREATE = "CHANNEL_CREATE"
    CHANNEL_UPDATE = "CHANNEL_UPDATE"
    CHANNEL_DELETE = "CHANNEL_DELETE"
    GROUP_CREATE = "GROUP_CREATE"
    GROUP_UPDATE = "GROUP_UPDATE"
    GROUP_DELETE = "GROUP_DELETE"
    USER_BAN = "USER_BAN"
    USER_UNBAN = "USER_UNBAN"
    MEMBER_JOIN = "MEMBER_JOIN"
    MEMBER_LEAVE = "MEMBER_LEAVE"
    COMMUNITY_UPDATE = "COMMUNITY_UPDATE"



import asyncio
import os
import swibots

from swibots.utils.types import DownloadProgressCallback
from swibots.error import SwitchError
from swibots.api.chat.models import Message
from typing import Union, Optional, Callable, BinaryIO

DOWNLOAD_MEDIA = "downloads/"


class DownloadMedia:
    async def download_media(self: "swibots.ApiClient", message: Message, file_name: str = DOWNLOAD_MEDIA, in_memory: bool = False, block: bool = True, progress: DownloadProgressCallback = None, progress_args: tuple = ()) -> Optional[Union[BinaryIO, bytes]]:
        """
        """

        # 1 = IMAGE, 2 = VIDEO, 3 = AUDIO, 7 = FILE
        if message.status not in [1, 2, 3, 7]:
            raise SwitchError("Message is not a media message")

        if message.media_link is None:
            raise SwitchError("Message does not have a media link")

        media_file_name = message.media_link.split("/")[-1]
        directory, file_name = os.path.split(file_name or "")
        file_name = file_name or media_file_name or ""

        download_fn = self.handle_download(
            url=message.media_link, directory=directory, file_name=file_name, in_memory=in_memory, block=block, progress=progress, progress_args=progress_args)
        if block:
            return await download_fn
        else:
            asyncio.get_event_loop().create_task(download_fn)



import swibots
from swibots.utils.types import JSONDict
from swibots.base import SwitchObject

from .base_typed_inline_query_result import BaseTypedInlineQueryResult
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup
from .types import InlineQueryResultType


class InlineQueryResultDocument(BaseTypedInlineQueryResult):
    def __init__(self,
                 id: str,
                 title: str,
                 document_url: str,
                 mime_type: str,
                 input_message: "InputMessageContent" = None,
                 description: str = None,
                 thumb_url: str = None,
                 thumb_width: int = None,
                 thumb_height: int = None,
                 reply_markup: "InlineMarkup" = None,
                 ):
        super().__init__(id, InlineQueryResultType.DOCUMENT, title, description, thumb_url, thumb_width, thumb_height,
                         input_message, reply_markup)
        self.document_url = document_url
        self.mime_type = mime_type

    def to_json(self) -> JSONDict:
        data = super().to_json()
        data.update({
            "documentUrl": self.document_url,
            "mimeType": self.mime_type,
        })
        return data



from .message import Message
from .inline_markup import InlineMarkup
from .inline_keyboard_button import InlineKeyboardButton
from .group_chat_history import GroupChatHistory
from .inline_keyboard_color import InlineKeyboardColor
from .inline_keyboard_size import InlineKeyboardSize
from .inline import *



from typing import TYPE_CHECKING, Optional, Type, TypeVar
import swibots
from swibots.api.chat.models import Message, InlineMarkup
from swibots.api.community.models import Channel, Group
from swibots.api.common.models import User
from swibots.api.common.models import MediaUploadRequest

if TYPE_CHECKING:
    from swibots.api import ApiClient


class SendText:
    async def send_text(self: "ApiClient", text: str, to: Optional[int | User] = None, channel: Optional[Channel | str] = None, group: Optional[Group | str] = None,  inline_markup: InlineMarkup = None, media: MediaUploadRequest = None) -> Message:
        """Send a message

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to send

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be sent

        This functions does the same as :meth:`~switch.api.chat.controllers.MessageController.send_message`.
        """
        return await self.chat_service.messages.send_text(text, to, channel, group, inline_markup, media)



import asyncio
import json
import logging
from typing import TYPE_CHECKING, List, Optional
from swibots.api.common.models import User, MediaUploadRequest
from swibots.api.chat.models import Message, GroupChatHistory, InlineMarkup, InlineQuery, InlineQueryAnswer
from swibots.error import SwitchError
from swibots.utils.types import JSONDict
from swibots.api.community.models import Channel, Community, Group

if TYPE_CHECKING:
    from swibots.api.chat import ChatClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/message"


class MessageController:
    """Message controller

    This controller is used to communicate with the message endpoints.

    """

    def __init__(self, client: "ChatClient"):
        self.client = client

    async def new_message(self, to: Optional[int | User] = None, channel: Optional[Channel | str] = None, group: Optional[Group | str] = None) -> Message:
        """Create a new message"""
        if isinstance(to, User):
            to = to.id

        if isinstance(channel, Channel):
            channel = channel.id

        if isinstance(group, Group):
            group = group.id

        return Message(
            user_id=self.client.user.id,
            receiver_id=to,
            channel_id=channel,
            group_id=group,
            app=self.client.app,
        )

    async def get_messages(self, user_id: int = None) -> List[Message]:
        """Get messages for a user

        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved
        """
        if user_id is None:
            user_id = self.client.user.id
        log.debug("Getting messages for user %s", user_id)
        response = await self.client.get(f"{BASE_PATH}/{user_id}")
        return self.client.build_list(Message, response.data)

    async def send_message(self, message: Message, media: MediaUploadRequest = None) -> Message:
        """Send a message

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to send
            media (``~switch.api.common.models.MediaUploadRequest``, *optional*): The media to send with the message

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be sent
        """
        data = message.to_json_request()
        log.debug("Sending message %s", json.dumps(data))

        if media:
            url = f"{BASE_PATH}/create-with-media"
            data.update(media.data_to_request())
            upload_fn = self.client.post(
                url, form_data=data, files=media.file_to_request(url))
            if media.block:
                response = await upload_fn
            else:
                asyncio.get_event_loop().create_task(upload_fn)
                return
        else:
            response = await self.client.post(f"{BASE_PATH}/create", data=data)
        return self.client.build_object(Message, response.data["message"])

    async def send_text(self, text: str, to: Optional[int | User] = None, channel: Optional[Channel | str] = None, group: Optional[Group | str] = None,  inline_markup: InlineMarkup = None, media: MediaUploadRequest = None) -> Message:
        """Send a message with text

        Parameters:
            to (``int`` | ``~switch.api.common.models.User``, *optional*): The user id to send the message to. Defaults to the current user id.
            text (``str``): The text to send
            channel (``~switch.api.community.models.Channel``, *optional*): The channel to send the message to
            group (``~switch.api.community.models.Group``, *optional*): The group to send the message to
            inline_markup (``~switch.api.chat.models.InlineMarkup``, *optional*): The inline markup to send with the message
            media (``~switch.api.common.models.MediaUploadRequest``, *optional*): The media to send with the message

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be sent
        """
        message = await self.new_message(to, channel, group)
        message.message = text
        message.inline_markup = inline_markup
        return await self.send_message(message, media)

    async def reply(self, message: int | Message, reply: Message, media: MediaUploadRequest = None) -> Message:
        if isinstance(message, Message):
            id = message.id
        else:
            id = message
        reply.replied_to_id = id
        return await self.send_message(reply, media)

    async def reply_text(self, message: int | Message, text: str, inline_markup: InlineMarkup = None, media: MediaUploadRequest = None) -> Message:
        """Reply to a message with text

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to reply to
            text (``str``): The text to reply with

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be sent
        """
        if isinstance(message, Message):
            id = message.id
        else:
            id = message

        m = Message(message=text, inline_markup=inline_markup)

        return await self.reply(id, m, media)

    async def edit_message(self, message: Message) -> Message:
        """Edit a message

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to edit

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be edited
        """
        data = message.to_json_request()
        log.debug("Editing message %s", json.dumps(data))
        response = await self.client.put(f"{BASE_PATH}?id={message.id}", data=data)
        return self.client.build_object(Message, response.data["message"])

    async def edit_message_text(self, message: int | Message, text: str, inline_markup: InlineMarkup = None) -> Message:
        """Edit a message with text

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to edit
            text (``str``): The text to edit with

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be edited
        """
        if isinstance(message, Message):
            id = message.id
        else:
            id = message
        return await self.edit_message(Message(id=id, message=text, inline_markup=inline_markup))

    async def delete_message(self, message: int | Message) -> bool:
        """Delete a message

        Parameters:
            message (``int`` | ``~switch.api.chat.models.Message``): The message id or message to delete

        Returns:
            ``bool``: True if the message was deleted

        Raises:
            ``~switch.error.SwitchError``: If the message could not be deleted
        """
        if isinstance(message, Message):
            id = message.id
        else:
            id = message
        log.debug("Deleting message %s", id)
        response = await self.client.delete(f"{BASE_PATH}/{id}")
        return True

    async def delete_messages_from_user(self, recipient_id: int, user_id: int = None) -> bool:
        """Delete messages from a user

        Parameters:
            recipient_id (``int``): The recipient id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``bool``: True if the messages were deleted

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be deleted

        """
        log.debug("Deleting messages for user %s", recipient_id)
        if user_id is None:
            user_id = self.client.user.id

        response = await self.client.delete(f"{BASE_PATH}/{user_id}/{recipient_id}")
        return True

    async def get_messages_between_users(
        self, recipient_id: int, user_id: int = None, page_limit: int = 100, page_offset: int = 0
    ) -> List[Message]:
        """Get messages between two users

        Parameters:
            recipient_id (``int``): The recipient id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            page_limit (``int``, *optional*): The page limit. Defaults to 100.
            page_offset (``int``, *optional*): The page offset. Defaults to 0.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved
        """
        q = []
        if page_limit:
            q.append(f"pageLimit={page_limit}")
        if page_offset:
            q.append(f"pageOffset={page_offset}")

        str_q = "&".join(q)

        if user_id is None:
            user_id = self.client.user.id

        log.debug("Getting messages for user %s", recipient_id)
        response = await self.client.get(f"{BASE_PATH}/{user_id}/{recipient_id}?{str_q}")
        return self.client.build_list(Message, response.data["messages"])

    async def forward_message(
        self,
        message: Message | int,
        group_channel: Group | Channel | str = None,
        receiver_id: str = None,
    ) -> Message:
        """Forward a message to a group or user

        Parameters:
            message (``~switch.api.chat.models.Message`` | ``int``): The message to forward
            group_channel (``~switch.api.chat.models.Group`` | ``~switch.api.chat.models.Channel`` | ``str``, *optional*): The group or channel to forward to. Defaults to None.
            receiver_id (``str``, *optional*): The user id to forward to. Defaults to None.

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be forwarded
        """
        if isinstance(message, Message):
            id = message.id
        else:
            id = message

        if isinstance(group_channel, Group):
            group_channel = group_channel.id
        elif isinstance(group_channel, Channel):
            group_channel = group_channel.id
        elif group_channel is not None:
            group_channel = group_channel

        q = []
        if group_channel is not None:
            q.append(f"groupChannelId={group_channel}")
        if receiver_id is not None:
            q.append(f"receiverId={receiver_id}")

        strQuery = q.join("&")

        log.debug("Forwarding message %s", id)
        response = await self.client.post(f"{BASE_PATH}/forward/{id}?{strQuery}")
        return self.client.build_object(Message, response.data["message"])
        # return Message.build_from_json(response.data["message"])

    async def get_message(self, message: int | Message) -> Message:
        """Get a message by id

        Parameters:
            message (``int`` | ``~switch.api.chat.models.Message``): The message id or message to get, if a message is passed, the id will be extracted from it.

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be retrieved
        """
        if isinstance(message, Message):
            id = message.id
        else:
            id = message
        log.debug("Getting message %s", id)
        response = await self.client.get(f"{BASE_PATH}/{id}")
        return self.client.build_object(Message, response.data["message"])
        # return Message.build_from_json(response.data["message"])

    async def get_group_chat_history(
        self,
        group_id: str,
        community_id: str,
        user_id: int = None,
        page_limit: int = 100,
        page_offset=0,
    ) -> GroupChatHistory:
        """Get group chat history

        Parameters:
            group_id (``str``): The group id
            community_id (``str``): The community id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            page_limit (``int``, *optional*): The page limit. Defaults to 100.
            page_offset (``int``, *optional*): The page offset. Defaults to 0.

        Returns:
            ``~switch.api.chat.models.GroupChatHistory``: The group chat history

        Raises:
            ``~switch.error.SwitchError``: If the group chat history could not be retrieved

        """
        log.debug("Getting group chat history for group %s", group_id)
        q = ["isChannel=false"]
        if page_limit:
            q.append(f"pageLimit={page_limit}")
        else:
            q.append(f"pageLimit=0")
        if page_offset:
            q.append(f"pageOffset={page_offset}")
        else:
            q.append(f"pageOffset=0")
        if community_id:
            q.append(f"communityId={community_id}")

        str_q = "&".join(q)

        if user_id is None:
            user_id = self.client.user.id

        response = await self.client.get(f"{BASE_PATH}/group/{user_id}/{group_id}?{str_q}")
        return self.client.build_object(GroupChatHistory, response.data)
        # return GroupChatHistory.build_from_json(response.data)

    async def get_channel_chat_history(
        self,
        channel_id: str,
        community_id: str,
        user_id: int = None,
        page_limit: int = 100,
        page_offset=0,
    ) -> GroupChatHistory:
        """Get channel chat history

        Parameters:
            channel_id (``str``): The channel id
            community_id (``str``): The community id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            page_limit (``int``, *optional*): The page limit. Defaults to 100.
            page_offset (``int``, *optional*): The page offset. Defaults to 0.

        Returns:
            ``~switch.api.chat.models.ChannelChatHistory``: The channel chat history

        Raises:
            ``~switch.error.SwitchError``: If the channel chat history could not be retrieved

        """
        log.debug("Getting channel chat history for channel %s", channel_id)
        q = ["isChannel=true"]
        if page_limit:
            q.append(f"pageLimit={page_limit}")
        else:
            q.append(f"pageLimit=0")
        if page_offset:
            q.append(f"pageOffset={page_offset}")
        else:
            q.append(f"pageOffset=0")
        if community_id:
            q.append(f"communityId={community_id}")

        str_q = "&".join(q)

        if user_id is None:
            user_id = self.client.user.id

        response = await self.client.get(f"{BASE_PATH}/group/{user_id}/{channel_id}?{str_q}")
        return self.client.build_object(GroupChatHistory, response.data)
        # return GroupChatHistory.build_from_json(response.data)

    async def get_community_media_files(self, community_id: str) -> List[Message]:
        """Get community media files

        Parameters:
            community_id (``str``): The community id

        Returns:
            ``List[~switch.api.chat.models.Message]``: The community media files

        Raises:
            ``~switch.error.SwitchError``: If the community media files could not be retrieved
        """
        log.debug("Getting community media files for community %s", community_id)
        response = await self.client.get(f"{BASE_PATH}/media?communityId={community_id}")
        return self.client.build_list(Message, response.data)
        # return Message.build_from_json_list(response.data)

    async def get_community_media_files_by_status(
        self, community_id: str, status: str
    ) -> List[Message]:
        """Get community media files by status

        Parameters:
            community_id (``str``): The community id
            status (``str``): The status of the media files

        Returns:
            ``List[~switch.api.chat.models.Message]``: The community media files


        Raises:
            ``~switch.error.SwitchError``: If the community media files could not be retrieved
        """
        log.debug("Getting community media files for community %s", community_id)
        response = await self.client.get(
            f"{BASE_PATH}/media?communityId={community_id}&status={status}"
        )
        return self.client.build_list(Message, response.data)
        # return Message.build_from_json_list(response.data)

    async def get_user_media_files(self, user_id: int = None) -> List[Message]:
        """Get user media files


        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The user media files

        Raises:
            ``~switch.error.SwitchError``: If the user media files could not be retrieved
        """
        if user_id is None:
            user_id = self.client.user.id
        log.debug("Getting user media files for user %s", user_id)
        response = await self.client.get(f"{BASE_PATH}/media/{user_id}")
        return self.client.build_list(Message, response.data)
        # return Message.build_from_json_list(response.data)

    async def clear_conversation(self, receiver_id: int) -> bool:
        """Clear a conversation

        Parameters:
            receiver_id (``int``): The receiver id

        Returns:
            ``bool``: True if the conversation was cleared

        Raises:
            ``~switch.error.SwitchError``: If the conversation could not be cleared
        """
        log.debug("Clearing conversation %s", receiver_id)
        response = await self.client.get(f"{BASE_PATH}/clearconversationwith/{receiver_id}")
        return True

    async def get_flag_messages(self, user_id: int = None) -> List[Message]:
        """Get flagged messages

        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The flagged messages

        Raises:
            ``~switch.error.SwitchError``: If the flagged messages could not be retrieved
        """
        if user_id is None:
            user_id = self.client.user.id

        log.debug("Get flag messages for %s", user_id)
        response = await self.client.get(f"{BASE_PATH}/flag?userId={user_id}")
        return self.client.build_list(Message, response.data)
        # return Message.build_from_json_list(response.data)

    async def flag_message(self, message: Message | int) -> bool:
        """Flag a message

        Parameters:
            message (``~switch.api.chat.models.Message`` | ``int``): The message to flag

        Returns:
            ``bool``: True if the message was flagged

        Raises:
            ``~switch.error.SwitchError``: If the message could not be flagged
        """
        if isinstance(message, Message):
            message_id = message.id
        else:
            message_id = message
        log.debug("Flagging message %s", message_id)
        response = await self.client.post(f"{BASE_PATH}/flag?messageId={message_id}")
        return True

    async def get_unread_messages_count(self, user_id: int = None) -> int:
        """Get unread messages

        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``int``: The unread messages count
        """
        if user_id is None:
            user_id = self.client.user.id

        log.debug("Get unread messages count for %s", user_id)
        response = await self.client.get(f"{BASE_PATH}/unread-messages?userId={user_id}")
        return response.data

    async def answer_inline_query(self, query: InlineQuery, answer: InlineQueryAnswer) -> bool:
        """Answer an inline query

        Parameters:
            query (``~switch.api.chat.models.InlineQuery``): The inline query
            answer (``~switch.api.chat.models.InlineQueryAnser``): The answer

        Returns:
            ``bool``: True if the query was answered

        Raises:
            ``~switch.error.SwitchError``: If the query could not be answered
        """
        if not isinstance(query, InlineQuery):
            raise TypeError("query must be an InlineQuery instance")

        if isinstance(answer, str):
            answer = InlineQueryAnswer(query_id=query.query_id, title=answer, results=[
            ], cache_time=0, is_personal=True, next_offset=None, pm_text=None, pm_parameter=None, user_id=query.user_id)

        if isinstance(answer, List):
            answer = InlineQueryAnswer(query_id=query.query_id, title=None, results=answer, cache_time=0,
                                       is_personal=True, next_offset=None, pm_text=None, pm_parameter=None, user_id=query.user_id)

        log.debug("Answering inline query %s", query.query_id)
        response = await self.client.post(f"{BASE_PATH}/inline/answer", answer.to_json_request())
        return response.data



def test_sum():
    assert 6 == 6, "Should be 6"


def test_sum_tuple():
    assert 6 == 6, "Should be 6"


if __name__ == "__main__":
    test_sum()
    test_sum_tuple()
    print("Everything passed")



import mimetypes
from swibots.utils.types import IOClient, ReadCallbackStream, UploadProgress, UploadProgressCallback


class MediaUploadRequest:
    def __init__(self,
                 path: str,
                 file_name: str = None,
                 mime_type: str = None,
                 caption: str = None,
                 description: str = None,
                 block: bool = True,
                 callback: UploadProgressCallback = None,
                 upload_args: tuple = ()):
        self.path = path
        self.file_name = file_name
        self.mime_type = mime_type
        self.caption = caption
        self.description = description
        self.block = block
        self.callback = callback
        self.upload_args = upload_args

    def data_to_request(self):
        return {
            "uploadMediaRequest.caption": self.caption,
            "uploadMediaRequest.description": self.description
        }

    def file_to_request(self, url):
        dProgress = UploadProgress(
            current=0,
            readed=0,
            file_name=self.file_name,
            client=IOClient(),
            url=url,
            callback=self.callback,
            args=self.upload_args
        )
        file = open(self.path, "rb")
        reader = ReadCallbackStream(
            file, dProgress.update
        )
        mime = self.mime_type or mimetypes.guess_type(
            self.path)[0] or "application/octet-stream"
        return {
            "uploadMediaRequest.file": (self.file_name or self.path, reader, mime)
        }



import re
from typing import TYPE_CHECKING, Optional, TypeVar
from swibots.api.chat.events import CallbackQueryEvent
from swibots.bots.filters.filter import Filter
from swibots.types import EventType

from swibots.utils.types import SCT, HandlerCallback
from swibots.bots.handlers.event_handler import EventHandler
from swibots.bots.bot_context import BotContext

if TYPE_CHECKING:
    pass

ResType = TypeVar("ResType")


class CallbackQueryHandler(EventHandler):
    def __init__(
        self,
        callback: HandlerCallback[BotContext[CallbackQueryEvent], ResType],
        filter: Optional[Filter] = None,
        **kwargs,
    ):
        super().__init__(EventType.CALLBACK_QUERY, callback, filter, **kwargs)

    async def should_handle(self, context: BotContext[CallbackQueryEvent]) -> bool:
        return (
            await super().should_handle(context)
            and context.event.callback_data is not None
            and context.event.message is not None
        )



from typing import TYPE_CHECKING, BinaryIO, Callable, List, Optional, Union
import swibots
from swibots.base import SwitchObject
from swibots.api.common import User, MediaUploadRequest, Media
from swibots.api.community import Community, Channel, Group
from swibots.utils.types import JSONDict
from .inline_markup import InlineMarkup


class Message(
    SwitchObject,
):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: int = None,
        user_id: int = None,
        user: "User" = None,
        receiver_id: int = None,
        receiver: "User" = None,
        message: str = None,
        sent_date: int = None,
        status: int = None,
        request_id: int = None,
        button_name: str = None,
        button_pressed_id: int = None,
        callback_data: str = None,
        channel_chat: bool = None,
        channel_id: int = None,
        channel: "Channel" = None,
        command_name: str = None,
        community_id: int = None,
        community: "Community" = None,
        edit: bool = None,
        flag: int = None,
        forward: bool = None,
        group_chat: bool = None,
        group_id: int = None,
        group: "Group" = None,
        information: str = None,
        inline_markup: "InlineMarkup" = None,
        is_read: bool = None,
        media_link: str = None,
        mentioned_ids: List[int] = None,
        personal_chat: bool = None,
        pinned: bool = None,
        reactions: List[str] = None,
        replied_message: str = None,
        replied_to_id: int = None,
        replied_to: "Message" = None,
        replies: List["Message"] = None,
        reply_count: int = None,
        media_id: int = None,
        media_info: Media = None,
        **kwargs,
    ):
        super().__init__(app=app)
        self.id = id
        self.user_id = user_id
        self.user = user
        self.receiver_id = receiver_id
        self.receiver = receiver
        self.message = message
        self.sent_date = sent_date
        self.status = status
        self.request_id = request_id
        self.button_name = button_name
        self.button_pressed_id = button_pressed_id
        self.callback_data = callback_data
        self.channel_chat = channel_chat
        self.channel_id = channel_id
        self.channel = channel
        self.command_name = command_name
        self.community_id = community_id
        self.community = community
        self.edit = edit
        self.flag = flag
        self.forward = forward
        self.group_chat = group_chat
        self.group_id = group_id
        self.group = group
        self.information = information
        self.inline_markup = inline_markup
        self.is_read = is_read
        self.media_link = media_link
        self.media_id = media_id
        self.media_info = media_info
        self.mentioned_ids = mentioned_ids
        self.personal_chat = personal_chat
        self.pinned = pinned
        self.reactions = reactions
        self.replied_message = replied_message
        self.replied_to_id = replied_to_id
        self.replied_to = replied_to
        self.replies = replies
        self.reply_count = reply_count
        self.__dict__.update(**kwargs)

    def to_json_request(self) -> JSONDict:
        return {
            "id": self.id,
            "message": self.message,
            "receiverId": self.receiver_id,
            "requestId": self.request_id,
            "userId": self.user_id,
            "communityId": self.community_id,
            "groupId": self.group_id,
            "channelId": self.channel_id,
            "inline_markup": self.inline_markup.to_json_request() if self.inline_markup else None,
            "callback_data": self.callback_data,
            "repliedTo": self.replied_to_id,
            "mediaLink": self.media_link,
            "status": self.status,
        }

    def to_json(self) -> JSONDict:
        return {
            "buttonName": self.button_name,
            "buttonPressedId": self.button_pressed_id,
            "callback_data": self.callback_data,
            "channelChat": self.channel_chat,
            "channelId": self.channel_id,
            "commandName": self.command_name,
            "communityId": self.community_id,
            "edit": self.edit,
            "flag": self.flag,
            "forward": self.forward,
            "groupChat": self.group_chat,
            "groupId": self.group_id,
            "id": self.id,
            "information": self.information,
            "inline_markup": self.inline_markup.to_json() if self.inline_markup else None,
            "isRead": self.is_read,
            "mediaLink": self.media_link,
            "mentionedIds": self.mentioned_ids,
            "message": self.message,
            "personalChat": self.personal_chat,
            "pinned": self.pinned,
            "reactions": self.reactions,
            "receiverId": self.receiver_id,
            "repliedMessage": self.replied_message,
            "repliedTo": self.replied_to_id,
            "replies": self.replies,
            "replyCount": self.reply_count,
            "requestId": self.request_id,
            "sentDate": self.sent_date,
            "status": self.status,
            "userId": self.user_id,
        }

    def from_json(self, data: Optional[JSONDict]) -> "Message":
        if data is not None:
            self.button_name = data.get("buttonName")
            self.button_pressed_id = data.get("buttonPressedId")
            self.callback_data = data.get("callback_data")
            self.channel_chat = data.get("channelChat")
            self.channel_id = data.get("channelId")
            self.channel = Channel.build_from_json(
                data.get("channel"), self.app)
            self.command_name = data.get("commandName")
            self.community_id = data.get("communityId")
            self.community = Community.build_from_json(
                data.get("community"), self.app)
            self.edit = data.get("edit")
            self.flag = data.get("flag")
            self.forward = data.get("forward")
            self.group_chat = data.get("groupChat")
            self.group_id = data.get("groupId")
            self.group = Group.build_from_json(data.get("group"), self.app)
            self.id = data.get("id")
            self.information = data.get("information")
            self.inline_markup = InlineMarkup.build_from_json(
                data.get("inline_markup"), self.app)
            self.is_read = data.get("isRead")
            self.media_link = data.get("mediaLink")
            self.media_id = data.get("mediaId")
            self.media_info = data.get("mediaInfo")
            self.mentioned_ids = data.get("mentionedIds")
            self.message = data.get("message")
            self.personal_chat = data.get("personalChat")
            self.pinned = data.get("pinned")
            self.reactions = data.get("reactions")
            self.receiver_id = data.get("receiverId")
            self.replied_to = data.get("repliedMessage")
            self.replied_to_id = data.get("repliedTo")
            self.replies = data.get("replies")
            self.reply_count = data.get("replyCount")
            self.request_id = data.get("requestId")
            self.sent_date = data.get("sentDate")
            self.status = data.get("status")
            self.user_id = data.get("userId")
        return self

    # async def get_receiver(self) -> "User":
    #     if self.receiver_id is None:
    #         return None
    #     if self._receiver is None:
    #         self._receiver = await self.app.get_user(self.receiver_id)
    #     return self._receiver

    # async def get_group(self) -> "Group":
    #     if self.group_id is None:
    #         return None
    #     if self.group is None:
    #         self.group = await self.app.getgroup(self.group_id)
    #     return self.group

    # async def get_channel(self) -> "Channel":
    #     if self.channel_id is None:
    #         return None
    #     if self.channel is None:
    #         self.channel = await self.app.get_channel(self.channel_id)
    #     return self.channel

    # async def get_community(self) -> "Community":
    #     if self.community_id is None:
    #         return None
    #     if self.community is None:
    #         self.community = await self.app.get_community(self.community_id)
    #     return self.community

    # async def get_replied_to(self) -> "Message":
    #     if self.replied_to_id is None or self.replied_to_id <= 0:
    #         return None
    #     if self.replied_to is None:
    #         self.replied_to = await self.app.get_message(self.replied_to_id)
    #     return self.replied_to

    async def get_replies(self) -> List["Message"]:
        if self.reply_count <= 0:
            return []
        if self.replies is None:
            self.replies = await self.app.get_message_replies(self.id)
        return self.replies

    async def get_replied_message(self) -> "Message":
        if self.replied_to_id is None or self.replied_to_id <= 0:
            return None
        if self.replied_to is None:
            self.replied_to = await self.app.get_message(self.replied_to_id)
        return self.replied_to

    ### API Methods ###

    async def send(self,  media: MediaUploadRequest = None) -> "Message":
        if self.id is not None:
            return await self.app.edit_message(self)
        return await self.app.send_message(self, media)

    async def delete(self) -> None:
        await self.app.delete_message(self)

    async def reply(self, reply: "Message", media: MediaUploadRequest = None) -> "Message":
        if isinstance(reply, str):
            return await self.app.reply_message_text(self, reply, media)
        return await self.app.reply_message(self, reply, media)

    async def reply_text(self, text: str, inline_markup: Optional[InlineMarkup] = None,  media: MediaUploadRequest = None) -> "Message":
        return await self.app.reply_message_text(self, text, inline_markup, media)

    async def edit_text(self, text: str,  inline_markup: Optional[InlineMarkup] = None) -> "Message":
        return await self.app.edit_message_text(self, text, inline_markup)

    async def download(self, file_name: str = None, in_memory: bool = False, block: bool = True, progress: Callable = None, progress_args: tuple = ()) -> Optional[Union[BinaryIO, bytes]]:
        await self.app.download_media(self, file_name, in_memory, block, progress, progress_args)



from typing import Callable, Optional
import swibots
from swibots.bots.filters.filter import Filter


class OnCallbackQuery:
    def on_callback_query(
        self: "swibots.BotApp" = None, filter: Optional[Filter] = None
    ) -> Callable:
        """Decorator for handling callback queries."""

        def decorator(func: Callable) -> Callable:
            if isinstance(self, swibots.BotApp):
                self.add_handler(swibots.bots.handlers.CallbackQueryHandler(func, filter))

            return func

        return decorator



from typing import TYPE_CHECKING, Optional
from swibots.api.api_client import ApiClient
from swibots.api.auth.models.auth_user import AuthUser
from swibots.api.bot.models import BotInfo, BotCommandInfo
from swibots.api.chat.models import Message

if TYPE_CHECKING:
    from swibots import BotApp


class Bot(AuthUser, ApiClient):
    def __init__(self, app: Optional["BotApp"] = None):
        super().__init__(app)
        self._info: BotInfo = None

    @property
    def app(self) -> "BotApp":
        return self._app

    @app.setter
    def app(self, value: "BotApp"):
        self._app = value
        # copy the api client
        self._chat_client = value.chat_service
        self._auth_client = value.auth_service
        self._community_client = value.community_service
        self._bot_client = value.bots_service

    async def on_app_start(self, app: "BotApp"):
        if app.auto_update_bot:
            """Called when app start
            This method registers the bot commands and updates the bot info
            """
            # get all app commands
            commands = self.app._register_commands or []
            description = self.app._bot_description or ""
            # register the commands
            self._info = BotInfo(description=description, id=self.id)
            for command in commands:
                command_name = command.command
                if isinstance(command_name, str):
                    command_names = command_name.split(",")
                else:
                    command_names = command_name

                for c_name in command_names:
                    self.info.commands.append(
                        BotCommandInfo(
                            command=c_name,
                            description=command.description,
                            channel=command.channel,
                        )
                    )

            self.info = await self.update_bot_info(self.info)

        pass

    @property
    def info(self) -> BotInfo:
        """Get the bot info"""
        return self._info

    @info.setter
    def info(self, value: BotInfo):
        """Set the bot info"""
        self._info = value

    @property
    def commands(self) -> list[BotCommandInfo]:
        """Get the bot commands"""
        return self._info.commands or []

    async def prepare_message(self, receiver_id: int, text: str, **kwargs) -> Message:
        """
        Prepares a message to be sent to the given receiver.

        Parameters:
            receiver_id (:obj:`int`): The receiver's id.
            text (:obj:`str`): The message's text.
            **kwargs: Additional keyword arguments to pass to the message constructor.

        Returns:
            :obj:`switch.api.chat.models.Message`: The prepared message.
        """
        message = Message(receiver_id=receiver_id,
                          message=text, app=self.app, **kwargs)
        message.user_id = self.id
        return message

    # async def prepare_response_message(self, message: Message) -> Message:
    #     """
    #     Prepares a message to be sent as a response to the given message.

    #     Parameters:
    #         message (:obj:`switch.api.chat.models.Message`): The message to respond to.

    #     Returns:
    #         :obj:`switch.api.chat.models.Message`: The prepared message.
    #     """

    #     receiver_id = message.user_id if message.user_id != self.id else message.receiver_id
    #     sender_id = self.id
    #     m = Message.build_from_json(message.to_json(), self.app)
    #     m.id = None
    #     m.message = ""

    #     if message.community_id is None or message.community_id == "":
    #         m.receiver_id = receiver_id

    #     m.user_id = sender_id
    #     return m

    async def prepare_response_message(self, message: Message) -> Message:
        """
        Prepares a message to be sent as a response to the given message.

        Parameters:
            message (:obj:`switch.api.chat.models.Message`): The message to respond to.

        Returns:
            :obj:`switch.api.chat.models.Message`: The prepared message.
        """

        response = Message(self.app)

        if message.community_id is not None and message.community_id != "":
            response.community_id = message.community_id
            response.group_id = message.group_id
            response.channel_id = message.channel_id
        else:
            receiver_id = message.user_id if message.user_id != self.id else message.receiver_id
            response.receiver_id = receiver_id

        response.user_id = self.id
        return response



from typing import TYPE_CHECKING, Type, TypeVar
import swibots
from swibots.api.chat.models import Message, InlineMarkup

if TYPE_CHECKING:
    from swibots.api import ApiClient


class EditMessageText:
    async def edit_message_text(self: "ApiClient", message: int | Message, text: str, inline_markup: InlineMarkup = None) -> Message:
        """Send a message

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to send

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be sent

        This functions does the same as :meth:`~switch.api.chat.controllers.MessageController.send_message`.
        """
        return await self.chat_service.messages.edit_message_text(message, text, inline_markup)



import asyncio
from contextlib import AbstractContextManager
from io import BytesIO
import logging
import mimetypes
import os
import re
import shutil
import signal
from signal import signal as signal_fn, SIGINT, SIGTERM, SIGABRT
from typing import Callable, Optional
import swibots
from swibots.api import ApiClient
from swibots.api.auth.models import AuthUser
from swibots.error import CancelError, SwitchError
from swibots.utils import RestClient, DownloadProgressCallback, UploadProgress, DownloadProgress, IOClient, ReadCallbackStream
import httpx

log = logging.getLogger(__name__)

# Signal number to name
signals = {
    k: v for v, k in signal.__dict__.items() if v.startswith("SIG") and not v.startswith("SIG_")
}


class App(AbstractContextManager, ApiClient):
    def __init__(
        self,
        # username: Optional[str] = None,
        # password: Optional[str] = None,
        token: Optional[str] = None,
        loop: asyncio.AbstractEventLoop = None,
    ):
        """Initialize the client"""
        super().__init__()
        self.token = token
        self._loop = loop or asyncio.get_event_loop()
        self._running = False
        self._user_type = AuthUser
        self.on_community_service_start: Callable = None
        self.on_chat_service_start: Callable = None
        self.on_app_stop: Callable = None
        self.on_app_start: Callable = None
        self.rest_client = RestClient()

    async def handle_upload(self, url, file_name, data=None, file_field="file", progress=None,  progress_args: tuple = ()):
        dProgress = UploadProgress(
            current=0,
            readed=0,
            file_name=file_name,
            client=IOClient(),
            url=file_name,
            callback=progress,
            callback_args=progress_args
        )
        file = open(file_name, "rb")
        reader = ReadCallbackStream(
            file, dProgress.update
        )
        mime = mimetypes.guess_type(file_name)[0] or "application/octet-stream"
        try:
            r = await self.rest_client._client.post(url, files={file_field: (file_name, reader, mime)}, data=data, headers={"Authorization": f"Bearer {self.token}"})
            return r.json()
        except CancelError:
            pass

    async def handle_download(self, url: str, file_name: str, directory="downloads/", in_memory: bool = False, block: bool = True, progress: DownloadProgressCallback = None, progress_args: tuple = ()):
        if directory is None or directory == "":
            directory = "downloads/"
        os.makedirs(directory, exist_ok=True) if not in_memory else None
        temp_file_path = os.path.abspath(
            re.sub("\\\\", "/", os.path.join(directory, file_name))) + ".temp"
        file = BytesIO() if in_memory else open(temp_file_path, "wb")

        dProgress = DownloadProgress(
            total=0,
            downloaded=0,
            file_name=file_name,
            client=IOClient(),
            url=url,
        )

        if (progress):
            await progress(dProgress, *progress_args)

        try:
            with httpx.stream("GET", url) as response:
                dProgress.total = int(response.headers["Content-Length"])
                dProgress.downloaded = response.num_bytes_downloaded
                dProgress.client = response
                dProgress.started = True
                for chunk in response.iter_bytes():
                    file.write(chunk)
                    dProgress.downloaded += len(chunk)
                    if progress:
                        await progress(dProgress, *progress_args)

        except BaseException as e:
            if not in_memory:
                file.close()
                os.remove(temp_file_path)
            if isinstance(e, CancelError):
                return None
            if isinstance(e, asyncio.CancelledError):
                raise e

            return None
        else:
            if in_memory:
                file.name = file_name
                return file
            else:
                file.close()
                file_path = os.path.splitext(temp_file_path)[0]
                shutil.move(temp_file_path, file_path)
                return file_path

        # file_id, directory, file_name, in_memory, file_size, progress, progress_args = packet

        # os.makedirs(directory, exist_ok=True) if not in_memory else None
        # temp_file_path = os.path.abspath(re.sub("\\\\", "/", os.path.join(directory, file_name))) + ".temp"
        # file = BytesIO() if in_memory else open(temp_file_path, "wb")

        # try:
        #     async for chunk in self.get_file(file_id, file_size, 0, 0, progress, progress_args):
        #         file.write(chunk)
        # except BaseException as e:
        #     if not in_memory:
        #         file.close()
        #         os.remove(temp_file_path)

        #     if isinstance(e, asyncio.CancelledError):
        #         raise e

        #     return None
        # else:
        #     if in_memory:
        #         file.name = file_name
        #         return file
        #     else:
        #         file.close()
        #         file_path = os.path.splitext(temp_file_path)[0]
        #         shutil.move(temp_file_path, file_path)
        #         return file_path

    async def _validate_credentials(self):
        if self.token is not None:
            return await self._validate_token()
        if self.username is None or self.password is None:
            raise SwitchError(
                "Username and password are required when token is not set")
        user = await self.login(user_type=self._user_type)
        self.user = user

    async def _validate_token(self):
        # check if token is valid
        if self.token is None:
            raise SwitchError("Token is not set")

        try:
            log.debug("checking token...")
            user = await self.get_me(user_type=self._user_type)
            self.user = user
            log.info("Logged in as [%s][%d]", user.user_name, user.id)
        except Exception as e:
            log.exception(e)
            await self.stop()
            raise SwitchError("Invalid token")

        if self.user is None:
            raise SwitchError("Invalid token")

    async def _validate_run(self):
        await self._validate_credentials()

    async def _on_app_stop(self):
        await self.chat_service.stop()
        await self.community_service.stop()
        if self.on_app_stop is not None:
            await self.on_app_stop(self)

    async def _on_app_start(self):
        if self.on_app_start is not None:
            await self.on_app_start(self)

    async def start(self):
        try:
            if self._running:
                raise SwitchError("App is already running")
            self._running = True
            """Starts the app"""
            log.info("🚀 Starting app...")

            await self._validate_run()

            try:
                await (self.chat_service.start())
                if self.on_chat_service_start is not None:
                    await self.on_chat_service_start(self)
            except Exception as e:
                log.exception(e)

            try:
                await (self.community_service.start())
                if self.on_community_service_start is not None:
                    await self.on_community_service_start(self)
            except Exception as e:
                log.exception(e)

            await self._on_app_start()

            log.info("🚀 App started!")

            # # run forever
            # while self._running:
            #     await asyncio.sleep(1)
        except asyncio.CancelledError:
            self._running = False
            # await self._do_stop()

    async def _do_stop(self):
        log.info("🛑 Stopping app...")
        await self._on_app_stop()
        self._running = False

    async def stop(self):
        if not self._running:
            return
        await self._do_stop()

    def __enter__(self):
        return self.start()

    def __exit__(self, *args):
        try:
            self.stop()
        except ConnectionError:
            pass

    async def __aenter__(self):
        return await self.start()

    async def __aexit__(self, *args):
        try:
            await self.stop()
        except ConnectionError:
            pass

    async def idle(self):
        task = None

        def signal_handler(signum, __):
            logging.info(
                f"Stop signal received ({signals[signum]}). Exiting...")
            task.cancel()

        for s in (SIGINT, SIGTERM, SIGABRT):
            signal_fn(s, signal_handler)

        while True:
            task = asyncio.create_task(asyncio.sleep(600))

            try:
                await task
            except asyncio.CancelledError:
                break

    def run(self, task: Callable = None):
        loop = asyncio.get_event_loop()
        run = loop.run_until_complete
        if task is not None:
            run(task)
        else:
            try:
                run(self.start())
                run(self.idle())
                run(self.stop())
            except KeyboardInterrupt:
                run(self.stop())
            except Exception as e:
                log.exception(e)
                run(self.stop())



from typing import Optional
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject


class Media(SwitchObject):
    def __init__(
        self,
        id: Optional[int] = None,
        caption: Optional[str] = None,
        description: Optional[str] = None,
        thumbnail_url: Optional[str] = None,
        source_id: Optional[bool] = None,
        media_type: Optional[bool] = None,
        mime_type: Optional[str] = None,
        file_name: Optional[bool] = None,
        file_size: Optional[bool] = None,
        url: Optional[bool] = None,
    ):
        self.id = id
        self.caption = caption
        self.description = description
        self.thumbnail_url = thumbnail_url
        self.source_id = source_id
        self.media_type = media_type
        self.mime_type = mime_type
        self.file_name = file_name
        self.file_size = file_size
        self.url = url

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "caption": self.caption,
            "description": self.description,
            "thumbnailUrl": self.thumbnail_url,
            "sourceId": self.source_id,
            "mediaType": self.media_type,
            "mimeType": self.mime_type,
            "fileName": self.file_name,
            "fileSize": self.file_size,
            "url": self.url,
        }

    def from_json(self, data: Optional[JSONDict] = None) -> "Media":
        if data is not None:
            self.id = data.get("id")
            self.caption = data.get("caption")
            self.description = data.get("description")
            self.thumbnail_url = data.get("thumbnailUrl")
            self.source_id = data.get("sourceId")
            self.media_type = data.get("mediaType")
            self.mime_type = data.get("mimeType")
            self.file_name = data.get("fileName")
            self.file_size = data.get("fileSize")
            self.url = data.get("url")
        return self



from .chat_event import ChatEvent
from .message_event import MessageEvent
from .command_event import CommandEvent
from .callback_query_event import CallbackQueryEvent
from .inline_query_event import InlineQueryEvent

__all__ = ["ChatEvent", "MessageEvent", "CommandEvent",
           "CallbackQueryEvent", "InlineQueryEvent"]



from typing import Optional
from swibots.base import SwitchObject
from swibots.utils.types import JSONDict


class BotCommandInfo(SwitchObject):
    def __init__(
        self,
        command: Optional[str] = None,
        description: Optional[str] = None,
        channel: Optional[bool] = False,
    ):
        self.bot_id = None
        self.command = command
        self.description = description
        self.channel = channel

    def from_json(self, data: JSONDict) -> "BotCommandInfo":
        if data is not None:
            self.bot_id = data.get("botId")
            self.command = data.get("command")
            self.description = data.get("description")
            self.channel = data.get("channel")
        return self

    def to_json(self) -> JSONDict:
        return {
            "botId": self.bot_id,
            "command": self.command,
            "description": self.description,
            "channel": self.channel,
        }



import swibots

from swibots.api.community.models import Community


class GetCommunity:
    async def get_community(self: "swibots.ApiClient", id: str) -> Community:
        """Get a community by its ID

        Args:
            id (`str`): The ID of the community

        Returns:
            :obj:`swibots.api.community.models.Community`: The community object.

        Raises:
            :obj:`switch.error.SwitchError`: If the community could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.CommunityController.get_community`.
        """
        return await self.community_service.communities.get_community(id)



from .rest_controller import RestController
from .rest_request import RestRequest
from .rest_response import RestResponse
from .switch_object import SwitchObject
from .switch_client import SwitchRestClient
from .switch_ws_async_client import SwitchWSAsyncClient



import logging
from typing import TYPE_CHECKING
from swibots.api.community.models import Group

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/group"


class GroupController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def get_group(self, group_id: str):
        """Get a channel by id"""
        response = await self.client.get(f"{BASE_PATH}?groupId={group_id}")
        return self.client.build_object(Group, response.data.get("result"))



import os


APP_CONFIG = {
    "CHAT_SERVICE": {
        "BASE_URL": os.getenv("CHAT_SERVICE_BASE_URL") or "http://51.159.11.53:9999",
        "WS_URL": os.getenv("CHAT_SERVICE_WS_URL")
        or "ws://51.159.11.53:9999/v1/websocket/message/ws",
    },
    "BOT_SERVICE": {
        "BASE_URL": os.getenv("BOT_SERVICE_BASE_URL") or "http://51.159.11.53:9999",
    },
    "AUTH_SERVICE": {
        "BASE_URL": os.getenv("AUTH_SERVICE_BASE_URL") or "http://51.159.11.53:9999/api",
    },
    "COMMUNITY_SERVICE": {
        "BASE_URL": os.getenv("COMMUNITY_SERVICE_BASE_URL") or "http://51.159.11.53:9999",
        "WS_URL": os.getenv("COMMUNITY_SERVICE_WS_URL")
        or "ws://51.159.11.53:9999/v1/websocket/community/ws",
    },
}


def get_config():
    return APP_CONFIG


def reload_config():
    APP_CONFIG["CHAT_SERVICE"]['BASE_URL'] = os.getenv(
        "CHAT_SERVICE_BASE_URL") or "http://51.159.11.53:9999"
    APP_CONFIG["CHAT_SERVICE"]['WS_URL'] = os.getenv(
        "CHAT_SERVICE_WS_URL") or "ws://51.159.11.53:9999/v1/websocket/message/ws"
    APP_CONFIG["BOT_SERVICE"]['BASE_URL'] = os.getenv(
        "BOT_SERVICE_BASE_URL") or "http://51.159.11.53:9999"
    APP_CONFIG["AUTH_SERVICE"]['BASE_URL'] = os.getenv(
        "AUTH_SERVICE_BASE_URL") or "http://51.159.11.53:9999/api"
    APP_CONFIG["COMMUNITY_SERVICE"]['BASE_URL'] = os.getenv(
        "COMMUNITY_SERVICE_BASE_URL") or "http://51.159.11.53:9999"
    APP_CONFIG["COMMUNITY_SERVICE"]['WS_URL'] = os.getenv(
        "COMMUNITY_SERVICE_WS_URL") or "ws://51.159.11.53:9999/v1/websocket/community/ws"


reload_config()



from typing import TYPE_CHECKING, Type, TypeVar
import swibots
from swibots.api.chat.models import Message

if TYPE_CHECKING:
    from swibots.api import ApiClient


class EditMessage:
    async def edit_message(self: "ApiClient", message: Message) -> Message:
        """Edit a message

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to edit

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be edited

        This method does the same as :meth:`~switch.api.chat.controllers.MessageController.edit_message`.
        """
        return await self.chat_service.messages.edit_message(message)



import json
from swibots.api.auth.models.auth_user import AuthUser
from swibots.api.community.events import *
from swibots.base import SwitchRestClient, SwitchWSAsyncClient
from swibots.config import get_config
from swibots.error import SwitchError
from swibots.utils.ws.asyncstomp.async_ws_subscription import AsyncWsSubscription
from swibots.utils.ws.common.ws_message import WsMessage
from swibots.types import EventType
from .controllers import ChannelController, GroupController, CommunityController
import swibots


class CommunityClient(SwitchRestClient):
    def __init__(
        self,
        app: "swibots.App" = None,
        base_url: str = None,
        ws_url: str = None,
    ):
        base_url = base_url or get_config()["COMMUNITY_SERVICE"]["BASE_URL"]
        self._ws_url = ws_url or get_config()["COMMUNITY_SERVICE"]["WS_URL"]
        super().__init__(app, base_url)
        self._channels = None
        self._groups = None
        self._communities = None
        self._ws: SwitchWSAsyncClient = None
        self._started = False

    @property
    def channels(self) -> ChannelController:
        """Get the channels controller"""
        if self._channels is None:
            self._channels = ChannelController(self)
        return self._channels

    @property
    def groups(self) -> GroupController:
        """Get the channels controller"""
        if self._groups is None:
            self._groups = GroupController(self)
        return self._groups

    @property
    def communities(self) -> CommunityController:
        """Get the channels controller"""
        if self._communities is None:
            self._communities = CommunityController(self)
        return self._communities

    @property
    def ws(self) -> SwitchWSAsyncClient:
        if self._ws is None:
            self._ws = SwitchWSAsyncClient(self._ws_url, self.token)
        return self._ws

    @property
    async def subscribe(self, endpoint: str, callback=None) -> AsyncWsSubscription:
        if self.user is None:
            raise SwitchError("User is not set")
        if callback is None:
            raise SwitchError("Callback is not set")
        return await self.ws.subscribe(endpoint, callback=callback)

    async def subscribe_to_notifications(self, callback=None) -> AsyncWsSubscription:
        subscription = await self.ws.subscribe(
            "/user/queue/events",
            callback=lambda event: callback(self._parse_event(event)),
        )
        return subscription

    def _parse_event(self, raw_message: WsMessage) -> CommunityEvent:
        json_data = json.loads(raw_message.body)
        type = json_data.get("type", "COMMUNITY")
        evt = None
        if type == EventType.CHANNEL_CREATE.value:
            evt = self.build_object(ChannelCreatedEvent, json_data)
            # return ChannelCreatedEvent.build_from_json(json_data)
        elif type == EventType.CHANNEL_UPDATE.value:
            evt = self.build_object(ChannelUpdatedEvent, json_data)
            # return ChannelUpdatedEvent.build_from_json(json_data)
        elif type == EventType.CHANNEL_DELETE.value:
            evt = self.build_object(ChannelDeletedEvent, json_data)
            # return ChannelDeletedEvent.build_from_json(json_data)
        elif type == EventType.COMMUNITY_UPDATE.value:
            evt = self.build_object(CommunityUpdatedEvent, json_data)
            # return CommunityUpdatedEvent.build_from_json(json_data)
        elif type == EventType.GROUP_CREATE.value:
            evt = self.build_object(GroupCreatedEvent, json_data)
            # return GroupCreatedEvent.build_from_json(json_data)
        elif type == EventType.GROUP_UPDATE.value:
            evt = self.build_object(GroupUpdatedEvent, json_data)
            # return GroupUpdatedEvent.build_from_json(json_data)
        elif type == EventType.GROUP_DELETE.value:
            evt = self.build_object(GroupDeletedEvent, json_data)
            # return GroupDeletedEvent.build_from_json(json_data)
        elif type == EventType.USER_BAN.value:
            evt = self.build_object(UserBannedEvent, json_data)
            # return UserBannedEvent.build_from_json(json_data)
        elif type == EventType.MEMBER_JOIN.value:
            evt = self.build_object(MemberJoinedEvent, json_data)
            # return MemberJoinedEvent.build_from_json(json_data)
        elif type == EventType.MEMBER_LEAVE.value:
            evt = self.build_object(MemberLeftEvent, json_data)
            # return MemberLeftEvent.build_from_json(json_data)
        else:
            evt = self.build_object(CommunityEvent, json_data)

        return evt

    async def start(self):
        """Start the community client"""
        if self.user is None:
            raise SwitchError("User is not set")
        await self.ws.connect()
        self._started = True

    async def stop(self):
        if self._started:
            await self.ws.disconnect()
            self._started = False



from swibots.api.auth.models.auth_user import AuthUser
from swibots.api.bot.controllers import BotController
from swibots.base import SwitchRestClient
from swibots.config import get_config
import swibots


class BotClient(SwitchRestClient):
    """Bot client

    This client is used to communicate with the bot service.

    Controllers:
        - :attr:`bots`: :obj:`~switch.api.bot.controllers.BotController` : The bot controller

    Properties:
        - :attr:`user`: :obj:`~switch.api.auth.models.auth_user.AuthUser` : The current user

    """

    def __init__(self, app: "swibots.App" = None, base_url: str = None):
        """Initialize the bot client

        Parameters:
            base_url (``str``): The base url of the bot service. Defaults to the value in the config.
        """
        base_url = base_url or get_config()["BOT_SERVICE"]["BASE_URL"]
        super().__init__(app, base_url)
        self._bots: BotController = None

    @property
    def bots(self) -> BotController:
        """Get the bot controller"""
        if self._bots is None:
            self._bots = BotController(self)
        return self._bots



from typing import TYPE_CHECKING, List, Optional
from swibots.api.common.models import User
from swibots.utils.types import JSONDict
from .bot_command_info import BotCommandInfo


class BotInfo(User):
    def __init__(
        self,
        id: Optional[str] = None,
        name: Optional[str] = None,
        username: Optional[str] = None,
        image_url: Optional[str] = None,
        active: Optional[bool] = None,
        deleted: Optional[bool] = None,
        role_info: Optional[str] = None,
        admin: Optional[bool] = None,
        is_bot: Optional[bool] = None,
        commands: Optional[List[BotCommandInfo]] = None,
        description: Optional[str] = None,
    ):
        super().__init__(
            id=id,
            name=name,
            username=username,
            image_url=image_url,
            active=active,
            deleted=deleted,
            role_info=role_info,
            admin=admin,
            is_bot=is_bot,
        )
        self.commands: List[BotCommandInfo] = commands or []
        self.description: Optional[str] = description

    def from_json(self, data: Optional[JSONDict] = None) -> "BotInfo":
        super().from_json(data)
        if data is not None:
            self.commands = [BotCommandInfo().from_json(x) for x in data.get("commands", [])]
            self.description = data.get("description")
        return self

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "commands": [x.to_json() for x in self.commands],
            "description": self.description,
        }



from typing import Generic, TypeVar

from swibots.utils.types import RequestMethod


T = TypeVar("T")


class RestRequest(Generic[T]):
    def __init__(
        self, path: str, method: RequestMethod = RequestMethod.GET, data=None, headers: dict = None
    ):
        self.method = method
        self.path = path
        self.data = data
        self.headers = headers



from typing import Optional
import swibots
from swibots.api.common.events.event import Event
from swibots.api.community.models.channel import Channel
from swibots.api.community.models.community import Community
from swibots.api.community.models.group import Group
from swibots.api.common.models.user import User
from swibots.api.chat.models.message import Message
from swibots.types import EventType
from swibots.utils.types import JSONDict
from .chat_event import ChatEvent


class MessageEvent(ChatEvent):
    """Message event"""

    def __init__(
        self,
        app: "swibots.App" = None,
        type: Optional[EventType] = None,
        community_id: Optional[str] = None,
        community: Optional[Community] = None,
        group_id: Optional[str] = None,
        group: Optional[Group] = None,
        channel_id: Optional[str] = None,
        channel: Optional[Channel] = None,
        action_by_id: Optional[str] = None,
        action_by: Optional[User] = None,
        data: Optional[dict] = None,
        user_id: Optional[str] = None,
        user: Optional[User] = None,
        message: Optional[Message] = None,
        message_id: Optional[int] = None,
    ):
        super().__init__(
            app=app,
            type=type or EventType.MESSAGE,
            community_id=community_id,
            community=community,
            group_id=group_id,
            group=group,
            channel_id=channel_id,
            channel=channel,
            action_by_id=action_by_id,
            action_by=action_by,
            data=data,
            user_id=user_id,
            user=user,
        )
        self.message_id = message_id
        self.message = message

    def from_json(self, data: JSONDict) -> "MessageEvent":
        super().from_json(data)
        if self.data is not None:
            self.message_id = data.get("messageId") or 0
            self.message: Message = Message.build_from_json(
                self.data.get("message"), self.app)

            self.message.user = self.user
            self.message.community = self.community
            self.message.group = self.group
            self.message.channel = self.channel
            self.message.receiver_id = self.message.receiver_id or self.data.get(
                "receiverId") or 0
            self.message.receiver = User.build_from_json(
                self.data.get("receiver") or {}, self.app)

        return self



from typing import List
import swibots
from swibots.api.chat.models import Message


class GetCommunityMediaFilesByStatus:
    async def get_community_media_files_by_status(
        self: "swibots.ApiClient", community_id: str, status: str
    ) -> List[Message]:
        """Get community media files by status

        Parameters:
            community_id (``int``): The community id
            status (``str``): The status

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_community_media_files_by_status`.
        """
        return await self.chat_service.messages.get_community_media_files_by_status(
            community_id, status
        )



from typing import TYPE_CHECKING, Generic, TypeVar
from swibots.api.api_client import ApiClient
from swibots.api.chat.models import Message
from .bot import Bot
from swibots.api.common.events import Event

EventType = TypeVar("EventType", bound="Event")


class BotContext(Generic[EventType], ApiClient):
    def __init__(self, bot: "Bot", event: EventType):
        self.event = event
        self.bot = bot
        # copy the api client
        self._chat_client = bot.chat_service
        self._auth_client = bot.auth_service
        self._community_client = bot.community_service
        self._bot_client = bot.bots_service

    async def prepare_message(self, receiver_id: int, text: str, **kwargs) -> Message:
        """
        Prepares a message to be sent to the given receiver.

        Parameters:
            receiver_id (:obj:`int`): The receiver's id.
            text (:obj:`str`): The message's text.
            **kwargs: Additional keyword arguments to pass to the message constructor.

        Returns:
            :obj:`switch.api.chat.models.Message`: The prepared message.
        """
        return await self.bot.prepare_message(receiver_id=receiver_id, text=text, **kwargs)

    async def prepare_response_message(self, message: Message) -> Message:
        """
        Prepares a message to be sent as a response to the given message.

        Parameters:
            message (:obj:`switch.api.chat.models.Message`): The message to respond to.

        Returns:
            :obj:`switch.api.chat.models.Message`: The prepared message.
        """
        return await self.bot.prepare_response_message(message=message)



from typing import TYPE_CHECKING, List, Optional
import swibots
from .inline_keyboard_color import InlineKeyboardColor
from .inline_keyboard_size import InlineKeyboardSize
from .inline_keyboard_button import InlineKeyboardButton
from swibots.base import SwitchObject

from swibots.utils.types import JSONDict

if TYPE_CHECKING:
    from .inline_keyboard_button import InlineKeyboardButton


class InlineMarkup(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        inline_keyboard: List[List["InlineKeyboardButton"]] = None,
        color: Optional[InlineKeyboardColor] = InlineKeyboardColor.RANDOM,
        size: Optional[InlineKeyboardSize] = InlineKeyboardSize.DEFAULT,
    ):
        super().__init__(app)
        self._inline_keyboard = inline_keyboard
        self._color = color or InlineKeyboardColor.RANDOM
        self._size = size or InlineKeyboardSize.DEFAULT

    @property
    def inline_keyboard(self) -> List[List["InlineKeyboardButton"]]:
        if self._inline_keyboard is None:
            self._inline_keyboard = []
        return self._inline_keyboard

    def add_row(self, buttons: List["InlineKeyboardButton"]):
        if len(buttons) > 0:
            self.inline_keyboard.append(list(buttons))

    def to_json(self) -> JSONDict:
        if self._inline_keyboard is None:
            return None
        return {
            "inlineKeyboard": [[x.to_json() for x in row] for row in self._inline_keyboard],
            "color": self._color.value,
            "size": self._size.value,
        }

    def from_json(self, data: JSONDict) -> "InlineMarkup":
        if data is None or data.get("inlineKeyboard") is None:
            return None
        self._color = InlineKeyboardColor(data.get("color") or "RANDOM")
        self._size = InlineKeyboardSize(data.get("size") or "DEFAULT")
        self._inline_keyboard = (
            [
                [InlineKeyboardButton.build_from_json(
                    x, self.app) for x in row]
                for row in data.get("inlineKeyboard") or []
            ],
        )

        return self



from typing import List, Optional

import swibots
from swibots.base import SwitchObject

from swibots.utils.types import JSONDict


class InlineKeyboardButton(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        text: Optional[str] = None,
        url: Optional[str] = None,
        callback_data: Optional[str] = None,
    ):
        super().__init__(app)
        self.text = text
        self.url = url
        self.callback_data = callback_data

    def to_json(self) -> JSONDict:
        return {
            "text": self.text,
            "url": self.url,
            "callbackData": self.callback_data,
        }

    def from_json(self, data: JSONDict) -> "InlineKeyboardButton":
        if data is not None:
            self.text = (data.get("text"),)
            self.url = (data.get("url"),)
            self.callback_data = (data.get("callbackData"),)
        return self



from typing import Generic, List, Optional, TypeVar
import swibots
from swibots.utils.types import JSONDict


T = TypeVar("T")


class SwitchObject(Generic[T]):
    def __init__(self, app:"swibots.App"=None, **kwargs):
        self._app = app
        for key, value in kwargs.items():
            setattr(self, key, value)

    @property
    def app(self) -> "swibots.App":
        return self._app

    @classmethod
    def build_from_json(cls, data: Optional[JSONDict] = None, app: Optional["swibots.App"] = None) -> Optional[T]:
        if data is None:
            return None
        return cls(app).from_json(data)

    @classmethod
    def build_from_json_list(cls, data: Optional[JSONDict], app: Optional["swibots.App"] = None) -> List[T]:
        return [cls.build_from_json(item, app) for item in data]

    def to_json_request(self) -> JSONDict:
        return self.to_json()

    def to_json(self) -> JSONDict:
        return self.__dict__

    def from_json(self, data: Optional[JSONDict]) -> T:
        for key, value in data.items():
            setattr(self, key, value)
        return self

    def __repr__(self) -> str:
        return self.to_json().__repr__()



from .bot_context import BotContext
from .handlers import *
from .constants import *
from .bot import Bot
from .register_command import RegisterCommand
from .filters import *
from .decorators import *



import re
from typing import TYPE_CHECKING, Any, Optional, TypeVar
from swibots.bots.constants import VALID_COMMAND_REGEX
from swibots.bots.filters.filter import Filter
from swibots.types import EventType

from swibots.utils.types import SCT, HandlerCallback
from swibots.bots.handlers.base_handler import BaseHandler
from swibots.bots.bot_context import BotContext

if TYPE_CHECKING:
    pass

ResType = TypeVar("ResType")


class UnknownCommandHandler(BaseHandler):
    def __init__(
        self,
        callback: HandlerCallback[BotContext[Any], ResType],
        filter: Optional[Filter] = None,
        **kwargs,
    ):
        super().__init__(callback, filter, **kwargs)

    async def should_handle(self, context: BotContext[Any]) -> bool:
        if context.event.type == EventType.COMMAND and context.event.message is not None:
            return True
        return False



from typing import Type, TypeVar
import swibots
from swibots.api.bot.models import BotInfo


class UpdateBotInfo:
    async def update_bot_info(self: "swibots.ApiClient", bot_info: BotInfo) -> BotInfo:
        """Update bot info

        Parameters:
            bot_info (``~switch.api.bot.models.BotInfo``): The bot info to update

        Returns:
            :obj:``~switch.api.bot.models.BotInfo``: The bot info

        This functions does the same as :meth:`~switch.api.bot.controllers.BotController.update_bot_info`.
        """
        return await self.bots_service.bots.update_bot_info(bot_info=bot_info)



import logging
from typing import TYPE_CHECKING
from swibots.api.community.models import Channel, Community, Group

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community"


class CommunityController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def get_community(self, community_id: str):
        """Get a community by id"""
        response = await self.client.get(f"{BASE_PATH}?communityId={community_id}")
        return self.client.build_object(Community, response.data.get("result"))



import swibots

from swibots.api.community.models import Channel


class GetChannel:
    async def get_channel(self: "swibots.ApiClient", id: str) -> Channel:
        """Get a community by its ID

        Args:
            id (`str`): The ID of the channel

        Returns:
            :obj:`swibots.api.community.models.Channel`: The channel object.

        Raises:
            :obj:`switch.error.SwitchError`: If the channel could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.ChannelController.get_channel`.
        """
        return await self.community_service.channels.get_channel(id)



from .types import InlineQueryResultType
import swibots
from swibots.base import SwitchObject
from swibots.utils.types import JSONDict
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup


class InlineQueryResult(SwitchObject):
    def __init__(self,
                 id: str,
                 type: InlineQueryResultType,
                 input_message: "InputMessageContent" = None,
                 reply_markup: "InlineMarkup" = None,
                 ):
        self.id = id
        self.type = type
        self.input_message = input_message
        self.reply_markup = reply_markup

    def to_json(self) -> JSONDict:
        data = {
            "id": self.id,
            "type": self.type.value,
        }
        if self.input_message is not None:
            data["inputMessage"] = self.input_message.to_json()
        if self.reply_markup is not None:
            data["replyMarkup"] = self.reply_markup.to_json()
        return data



import logging
import swibots
from typing import TYPE_CHECKING, Type, TypeVar
from ..models import AuthUser

if TYPE_CHECKING:
    from swibots.api.auth import AuthClient

log = logging.getLogger(__name__)

BASE_PATH = "/user"
T = TypeVar("T", bound="AuthUser")


class UserController:
    """User controller

    This controller is used to communicate with the user endpoints.

    """

    def __init__(self, client: "AuthClient"):
        self.client = client

    async def me(self, user_type: Type[T] = AuthUser) -> T:
        """Get the current user

        Parameters:
            user_type (``Type[T]``, *optional*): The user type to return. Defaults to :obj:`~switch.api.auth.models.AuthUser`.

        Returns:
            ``T``: The current user

        """
        response = await self.client.get(f"{BASE_PATH}")
        return self.client.build_object(user_type, response.data)
    
    async def login(self, username: str, password: str, user_type: Type[T] = AuthUser) -> T:
        """Login with username and password

        Parameters:
            username (``str``): The username
            password (``str``): The password
            user_type (``Type[T]``, *optional*): The user type to return. Defaults to :obj:`~switch.api.auth.models.AuthUser`.

        Returns:
            ``T``: The user

        """
        response = await self.client.post(f"{BASE_PATH}/login", json={"username": username, "password": password})
        return self.client.build_object(user_type, response.data)



from .delete_bot_info import DeleteBotInfo
from .get_bot_info import GetBotInfo
from .update_bot_info import UpdateBotInfo


class BotMethods(
    DeleteBotInfo,
    GetBotInfo,
    UpdateBotInfo,
):
    pass



import swibots

from swibots.api.community.models import Group


class GetGroup:
    async def get_group(self: "swibots.ApiClient", id: str) -> Group:
        """Get a community by its ID

        Args:
            id (`str`): The ID of the group

        Returns:
            :obj:`swibots.api.community.models.Group`: The group object.

        Raises:
            :obj:`switch.error.SwitchError`: If the group could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.GroupController.get_group`.
        """
        return await self.community_service.groups.get_group(id)



from typing import Type, TypeVar
import swibots
from swibots.api.auth.models import AuthUser

T = TypeVar("T", bound="swibots.AuthUser")


class GetMe:
    async def get_me(self: "swibots.ApiClient", user_type: Type[T] = AuthUser) -> T:
        """Get the current user

        Parameters:
            user_type (``Type[T]``, *optional*): The user type to return. Defaults to :obj:`~switch.api.auth.models.AuthUser`.

        Returns:
            ``T``: The current user

        This functions does the same as :meth:`~switch.api.auth.controllers.UserController.me`.

        """
        return await self.auth_service.users.me(user_type=user_type)



import json
import logging
from typing import Tuple
from swibots.api.auth.models.auth_user import AuthUser
from swibots.api.chat.controllers import MessageController
from swibots.api.chat.events import ChatEvent, CallbackQueryEvent, MessageEvent, CommandEvent, InlineQueryEvent
from swibots.base import SwitchRestClient, SwitchWSAsyncClient
from swibots.config import get_config
from swibots.error import SwitchError
from swibots.types import EventType
from swibots.utils.ws.asyncstomp.async_ws_subscription import AsyncWsSubscription
from swibots.utils.ws.common.ws_message import WsMessage
import swibots

logger = logging.getLogger(__name__)


class ChatClient(SwitchRestClient):
    """Chat client

    This client is used to communicate with the chat service.

    Controllers:
        - :attr:`messages`: :obj:`~switch.api.chat.controllers.MessageController` : The message controller

    Properties:
        - :attr:`user`: :obj:`~switch.api.auth.models.auth_user.AuthUser` : The current user
        - :attr:`ws`: :obj:`~switch.base.SwitchWSAsyncClient` : The websocket client

    """

    def __init__(
        self,
        app: "swibots.App" = None,
        base_url: str = None,
        ws_url: str = None,
    ):
        """Initialize the chat client

        Parameters:
            base_url (``str``): The base url of the chat service. Defaults to the value in the config.
            ws_url (``str``): The websocket url of the chat service. Defaults to the value in the config.
        """
        base_url = base_url or get_config()["CHAT_SERVICE"]["BASE_URL"]
        self._ws_url = ws_url or get_config()["CHAT_SERVICE"]["WS_URL"]
        super().__init__(app, base_url)
        self._messages: MessageController = None
        self._ws: SwitchWSAsyncClient = None
        self._started = False

    @property
    def ws(self) -> SwitchWSAsyncClient:
        if self._ws is None:
            self._ws = SwitchWSAsyncClient(self._ws_url, self.token)
        return self._ws

    @property
    def messages(self) -> MessageController:
        """Get the message controller"""
        if self._messages is None:
            self._messages = MessageController(self)
        return self._messages

    async def subscribe(self, endpoint: str, callback=None) -> AsyncWsSubscription:
        """Subscribe to a websocket endpoint

        Parameters:
            endpoint (``str``): The endpoint to subscribe to
            callback (``callable``): The callback to call when a message is received

        Returns:
            :obj:`~switch.utils.ws.asyncstomp.async_ws_subscription.AsyncWsSubscription`: The subscription

        Raises:
            :obj:`~switch.error.SwitchError`: If the user is not set or the callback is not set
        """
        if self.user is None:
            raise SwitchError("User is not set")
        if callback is None:
            raise SwitchError("Callback is not set")
        return await self.ws.subscribe(endpoint, callback=callback)

    async def subscribe_to_notifications(self, callback=None) -> AsyncWsSubscription:
        """Subscribe to the notification endpoint

        Parameters:
            callback (``callable``): The callback to call when a message is received

        Returns:
            :obj:`~switch.utils.ws.asyncstomp.async_ws_subscription.AsyncWsSubscription`: The subscription

        Raises:
            :obj:`~switch.error.SwitchError`: If the user is not set or the callback is not set

        This is a shortcut for :meth:`subscribe` with the endpoint set to ``/chat/queue/events``
        """
        subscription = await self.ws.subscribe(
            "/chat/queue/events",
            callback=lambda event: callback(self._parse_event(event)),
        )
        return subscription

    def _parse_event(self, raw_message: WsMessage) -> ChatEvent:
        try:
            json_data = json.loads(raw_message.body)
            type = json_data.get("type", "MESSAGE")
            evt: ChatEvent = None
            if type == EventType.MESSAGE.value:
                evt = self.build_object(MessageEvent, json_data)
                # evt = MessageEvent.build_from_json(json_data)
            elif type == EventType.COMMAND.value:
                evt = self.build_object(CommandEvent, json_data)
                # evt = CommandEvent.build_from_json(json_data)
            elif type == EventType.CALLBACK_QUERY.value:
                evt = self.build_object(CallbackQueryEvent, json_data)
                # evt = CallbackQueryEvent.build_from_json(json_data)
            elif type == EventType.INLINE_QUERY.value:
                evt = self.build_object(InlineQueryEvent, json_data)
            else:
                evt = self.build_object(ChatEvent, json_data)
                # evt = ChatEvent.build_from_json(json_data)
            return evt
        except Exception as e:
            logger.exception(e)

    async def start(self):
        """Start the chat websocket client
        Raises:
            :obj:`~switch.error.SwitchError`: If the user is not set
        """
        if self.user is None:
            raise SwitchError("User is not set")
        await self.ws.connect()
        self._started = True

    async def stop(self):
        """Stop the chat websocket client"""
        if self._started:
            await self.ws.disconnect()
            self._started = False



from .inline_query import InlineQuery
from .inline_query_answer import InlineQueryAnswer
from .inline_query_result import InlineQueryResult
from .types import InlineQueryResultType
from .base_typed_inline_query_result import BaseTypedInlineQueryResult
from .inline_query_result_article import InlineQueryResultArticle
from .inline_query_result_photo import InlineQueryResultPhoto
from .inline_query_result_document import InlineQueryResultDocument
from .inline_query_result_video import InlineQueryResultVideo
from .input_message_content import InputMessageContent



from swibots.base import SwitchObject
from typing import List

from .message import Message
from swibots.api.common.models.user import User
import swibots


class GroupChatHistory(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        users: List[User] = None,
        messages: List[Message] = None,
    ):
        super().__init__(app)
        self.users = users or []
        self.messages = messages or []

    def from_json(self, data: dict) -> "GroupChatHistory":
        self.users = User.build_from_json_list(
            data.get("userInfo", []), self.app)
        self.messages = Message.build_from_json_list(
            data.get("message", []),  self.app)
        return self

    def to_json(self) -> dict:
        return {
            "userInfo": [user.to_json() for user in self.users],
            "message": [message.to_json() for message in self.messages],
        }



from typing import Optional
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject
import swibots

class Community(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: Optional[str] = None,
        name: Optional[str] = None,
        username: Optional[str] = None,
        profile_url: Optional[str] = None,
        cover_url: Optional[str] = None,
        is_public: Optional[bool] = None,
        is_free: Optional[bool] = None,
        created_by: Optional[str] = None,
        guidelines: Optional[str] = None,
        description: Optional[str] = None,
        verified: Optional[bool] = None,
        category: Optional[str] = None,
        type: Optional[str] = None,
        link: Optional[str] = None,
        icon: Optional[str] = None,
    ):
        super().__init__(app)
        self.id = id
        self.name = name
        self.username = username
        self.profile_url = profile_url
        self.cover_url = cover_url
        self.is_public = is_public
        self.is_free = is_free
        self.created_by = created_by
        self.guidelines = guidelines
        self.description = description
        self.verified = verified
        self.category = category
        self.type = type
        self.link = link
        self.icon = icon

    def to_json(self) -> JSONDict:
        return {
            "communityId": self.id,
            "communityName": self.name,
            "communityUsername": self.username,
            "communityProfileUrl": self.profile_url,
            "communityCoverUrl": self.cover_url,
            "isPublic": self.is_public,
            "isFree": self.is_free,
            "createdBy": self.created_by,
            "communityGuidelines": self.guidelines,
            "communityDescription": self.description,
            "verified": self.verified,
            "communityCategory": self.category,
            "communityType": self.type,
            "link": self.link,
            "icon": self.icon,
        }

    def from_json(self, data: Optional[JSONDict]) -> Optional["Community"]:
        if data is not None:
            self.id = data.get("communityId")
            self.name = data.get("communityName")
            self.username = data.get("communityUsername")
            self.profile_url = data.get("communityProfileUrl")
            self.cover_url = data.get("communityCoverUrl")
            self.is_public = data.get("isPublic")
            self.is_free = data.get("isFree")
            self.created_by = data.get("createdBy")
            self.guidelines = data.get("communityGuidelines")
            self.description = data.get("communityDescription")
            self.verified = data.get("verified")
            self.category = data.get("communityCategory")
            self.type = data.get("communityType")
            self.link = data.get("link")
            self.icon = data.get("icon")
        return self



import asyncio
import logging
from typing import List, Collection, Optional
import swibots
from swibots.api.community.events import CommunityEvent
from swibots.api.chat.events import ChatEvent
from swibots.bots import BotContext, Decorators, BaseHandler
from swibots.api.common.events import Event
from .app import App

log = logging.getLogger(f"{__name__}")


class BotApp(App, Decorators):
    """Bot client

    This is the main class for interacting with the Switch BOT API.

    """

    def __init__(
        self,
        # username: Optional[str] = None,
        # password: Optional[str] = None,
        token: Optional[str] = None,
        bot_description: Optional[str] = None,
        auto_update_bot: Optional[bool] = True,
        loop: asyncio.AbstractEventLoop = None,
    ):
        """
        Initialize the client

        Args:
            token (:obj:`str`): The bot token.
            bot_description(:obj:`str`): The bot description.
            auto_update_bot(:obj:`bool`): Whether to automatically update the bot description and the regitered commands.
            loop (:obj:`asyncio.AbstractEventLoop`): The asyncio loop to use (default: asyncio.get_event_loop()).

        """
        super().__init__(token, loop)
        self._user_type = swibots.bots.Bot
        self.on_chat_service_start = self._on_chat_service_start
        self.on_community_service_start = self._on_community_service_start
        self._handlers: List[BaseHandler] = []
        self._register_commands: List[swibots.bots.RegisterCommand] = []
        self._bot_description = bot_description
        self.auto_update_bot = auto_update_bot

    @property
    def bot(self) -> "swibots.bots.Bot":
        """
        The bot user.

            Returns:
                :obj:`swibots.bots.Bot`: The bot user.
        """
        return self.user

    @property
    def handlers(self) -> List[BaseHandler]:
        """
        Get the list of handlers.

        Returns:
            :obj:`List[BaseHandler]`: The list of handlers.
        """
        if self._handlers is None:
            self._handlers = []
        return self._handlers

    def register_command(
        self, command: swibots.bots.RegisterCommand | List[swibots.bots.RegisterCommand]
    ) -> "BotApp":
        if self._running:
            raise swibots.SwitchError(
                "Cannot register commands after the app has started")
        if isinstance(command, list):
            self._register_commands.extend(command)
        else:
            self._register_commands.append(command)
        return self

    def add_handler(self, handler: BaseHandler | List[BaseHandler]) -> "BotApp":
        if isinstance(handler, list):
            self.handlers.extend(handler)
        else:
            self.handlers.append(handler)
        return self

    async def _validate_token(self):
        await super()._validate_token()
        if not isinstance(self.user, swibots.Bot):
            raise swibots.SwitchError("Invalid token")

        if not self.user.is_bot:
            raise swibots.SwitchError("Invalid token (not a bot)")

        self.user.app = self
        # Register commands
        await self.user.on_app_start(self)

    async def _on_chat_service_start(self, _):
        await self.chat_service.subscribe_to_notifications(callback=self.on_chat_event)

    async def _on_community_service_start(self, _):
        await self.community_service.subscribe_to_notifications(callback=self.on_community_event)

    def _build_context(self, event: Event) -> BotContext:
        return BotContext(bot=self.bot, event=event)

    async def process_event(self, ctx: BotContext):
        for handler in self.handlers:
            if await handler.should_handle(ctx):
                try:
                    await handler.handle(ctx)
                except Exception as e:
                    log.exception(f"Error while processing event: {e}")
                    raise e
                finally:
                    break

    async def on_community_event(self, evt: CommunityEvent):
        if evt is not None and isinstance(evt, Event):
            await self.process_event(self._build_context(evt))

    async def on_chat_event(self, evt: ChatEvent):
        if evt is not None:
            await self.process_event(self._build_context(evt))



import json
import logging
from typing import TYPE_CHECKING, List
from swibots.api.bot.models import BotInfo, BotCommandInfo
from swibots.error import SwitchError
from swibots.utils.types import JSONDict

if TYPE_CHECKING:
    from swibots.api.bot import BotClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/bots"


class BotController:
    """Bot controller

    This controller is used to communicate with the bot endpoints.
    """

    def __init__(self, client: "BotClient"):
        self.client = client

    async def get_bot_info(self, bot_id: str) -> BotInfo:
        """Get bot info

        Parameters:
            bot_id (``str``): The bot id. Defaults to the current bot id.

        Returns:
            :obj:``~switch.api.bot.models.BotInfo``: The bot info
        """
        if bot_id is None:
            bot_id = self.client.user.id
        response = await self.client.get(f"{BASE_PATH}/{bot_id}")
        return BotInfo.build_from_json(response.data)

    async def update_bot_info(self, bot_info: BotInfo) -> BotInfo:
        """Update bot info

        Parameters:
            bot_info (``~switch.api.bot.models.BotInfo``): The bot info to update

        Returns:
            :obj:``~switch.api.bot.models.BotInfo``: The bot info
        """
        data = bot_info.to_json_request()
        response = await self.client.put(f"{BASE_PATH}", data=data)
        return BotInfo.build_from_json(response.data)

    async def delete_bot_info(self, bot_id: str) -> bool:
        """Delete bot info

        Parameters:
            bot_id (``str``): The bot id. Defaults to the current bot id.

        Returns:
            ``bool``: True if the bot was deleted
        """
        response = await self.client.delete(f"{BASE_PATH}/{bot_id}")
        return True



from swibots.utils.ws.common import WsMessage


class AsyncWsSubscription:
    def __init__(
        self, client, destination: str, id: str, headers: dict[str, str] = None, callback=None
    ):
        from .async_ws_client import AsyncWsClient

        self.client: AsyncWsClient = client
        self.destination = destination
        self.callback = callback
        self.id = id
        self.headers = headers or {}

    async def start(self):
        self.headers["id"] = self.id
        self.headers["destination"] = self.destination
        await self.client._transmit("SUBSCRIBE", self.headers)

    async def receive(self, message):
        if self.callback is not None:
            await self.callback(WsMessage(message))

    async def send(self, body: str, headers: dict[str, str] = None):
        headers = headers or {}
        await self.client.send(self.destination, headers, body)

    async def unsubscribe(self):
        await self.client.unsubscribe(self.id)



from swibots.utils.types import SCT


class RegisterCommand:
    def __init__(
        self,
        command: SCT[str],
        description: str,
        channel: bool = False,
    ):
        self.command = command
        self.description = description
        self.channel = channel



from enum import Enum
from typing import Any, Callable, Collection, Coroutine, Dict, TypeVar, Union

from swibots.error import CancelError


class IOClient:
    def cancel(self) -> None:
        raise CancelError()


class DownloadProgress:
    def __init__(self, downloaded: int, total: int, url: str, client: IOClient, file_name: str):
        self.downloaded = downloaded
        self.total = total
        self.url = url
        self.client = client
        self.file_name = file_name
        self.started = False


class UploadProgress:
    def __init__(self, current: int, readed: int, url: str, client: IOClient, file_name: str, callback, callback_args):
        self.current = current
        self.readed = readed
        self.url = url
        self.client = client
        self.file_name = file_name
        self.started = False
        self.callback = callback
        self.callback_args = callback_args

    def update(self, current: int) -> None:
        self.current = current
        self.readed += current
        if self.callback:
            self.callback(self, *self.callback_args)


CtxType = TypeVar("CtxType")
ResType = TypeVar("ResType")
HandlerCallback = Callable[[CtxType], Coroutine[Any, Any, ResType]]
FilterCallback = Callable[[CtxType], Coroutine[Any, Any, bool]]
DownloadProgressCallback = Callable[[
    DownloadProgress], Coroutine[Any, Any, None]]
UploadProgressCallback = Callable[[
    DownloadProgress], Coroutine[Any, Any, None]]

RT = TypeVar("RT")
SCT = Union[RT, Collection[RT]]
"""Single instance or collection of instances."""

JSONDict = Dict[str, Any]


class ReadCallbackStream(object):
    """Wraps a file-like object in another, but also calls a user
    callback with the number of bytes read whenever its `read()` method
    is called. Used for tracking upload progress, for example for a
    progress bar in a UI application. Idea taken from ActiveState Code Recipe:
    http://code.activestate.com/recipes/578669-wrap-a-string-in-a-file-like-object-that-calls-a-u/
    """

    def __init__(self, file_like, callback):
        self.file_like = file_like
        self.callback = callback

    def __len__(self):
        raise NotImplementedError()

    def read(self, *args):
        chunk = self.file_like.read(*args)
        if len(chunk) > 0:
            self.callback(len(chunk))
        return chunk


class RequestMethod(Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"



import re
from typing import TYPE_CHECKING, Generic, Optional, TypeVar
from swibots.bots.constants import VALID_COMMAND_REGEX
from swibots.bots.filters.filter import Filter
from swibots.types import EventType

from swibots.utils.types import SCT, HandlerCallback
from swibots.bots.handlers.base_handler import BaseHandler
from swibots.bots.bot_context import BotContext
from swibots.api.chat.events import CommandEvent
from swibots.api.common.events import Event

if TYPE_CHECKING:
    pass

ResType = TypeVar("ResType")


class EventHandler(BaseHandler):
    def __init__(
        self,
        event_types: Optional[SCT[EventType]],
        callback: HandlerCallback[BotContext[Event], ResType],
        filter: Optional[Filter] = None,
        **kwargs,
    ):
        super().__init__(callback, filter, **kwargs)
        if isinstance(event_types, EventType):
            event_types = frozenset({event_types})
        else:
            event_types = frozenset(event_types)
        self.event_types = event_types
        self.filter = filter

    async def should_handle(self, context: BotContext[Event]) -> bool:
        if (self.event_types is not None) and (not context.event.type in self.event_types):
            return False
        if self.filter:
            return await self.filter(context)
        return True



from typing import List
import swibots
from swibots.api.chat.models import Message


class GetMessage:
    async def get_message(self: "swibots.ApiClient", message_id: int) -> Message:
        """Get a message

        Parameters:
            message_id (``int``): The message id

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_message`.
        """
        return await self.chat_service.messages.get_message(message_id)



import re
from typing import Optional
from swibots.api.chat.events import MessageEvent
from swibots.api.chat.events.chat_event import ChatEvent
from swibots.api.common.events import Event
from swibots.bots.bot_context import BotContext
from swibots.utils.types import SCT, FilterCallback
from swibots.types import EventType


class Filter:
    async def __call__(self, ctx: BotContext) -> bool:
        raise NotImplementedError

    def __invert__(self):
        return InvertFilter(self)

    def __and__(self, other):
        return AndFilter(self, other)

    def __or__(self, other):
        return OrFilter(self, other)


class InvertFilter(Filter):
    def __init__(self, base):
        self.base = base

    async def __call__(self, ctx: BotContext) -> bool:
        result = await self.base(ctx)
        return not result


class AndFilter(Filter):
    def __init__(self, base, other):
        self.base = base
        self.other = other

    async def __call__(self, ctx: BotContext) -> bool:
        r1 = await self.base(ctx)
        # short circuit
        if not r1:
            return False
        r2 = await self.other(ctx)

        return r1 and r2


class OrFilter(Filter):
    def __init__(self, base, other):
        self.base = base
        self.other = other

    async def __call__(self, ctx: BotContext) -> bool:
        r1 = await self.base(ctx)
        # short circuit
        if r1:
            return True

        return await self.other(ctx)


CUSTOM_FILTER_NAME = "CustomFilter"


def create(func: FilterCallback, name: str = None, **kwargs) -> Filter:
    return type(
        name or func.__name__ or CUSTOM_FILTER_NAME, (Filter,), {
            "__call__": func, **kwargs}
    )()


async def all_filter(self, ctx: BotContext):
    return True


all = create(all_filter)
"""Filter all messages."""


async def self_filter(self, ctx: BotContext[MessageEvent]):
    return bool(
        ctx.event.message is not None
        and ctx.event.message.user_id == ctx.event.message.receiver_id
    )


self = create(self_filter)
"""Filter messages generated by the same user."""


async def bot_filter(self, ctx: BotContext[MessageEvent]):
    return bool(
        ctx.event.message is not None and ctx.event.user is not None and ctx.event.user.is_bot
    )


is_bot = create(bot_filter)
"""Filter messages coming from bots."""


async def me_filter(self, ctx: BotContext[MessageEvent]):
    return bool(ctx.event.message is not None and ctx.event.message.user_id == ctx.user.id)


me = create(me_filter)
"""Filter messages coming from your user."""


async def incoming_filter(self, ctx: BotContext[MessageEvent]):
    return bool(ctx.event.message is not None and ctx.event.message.receiver_id == ctx.user.id)


incoming = create(incoming_filter)
"""Filter incoming messages. Messages that are sent to the users recipient."""


async def outgoing_filter(self, ctx: BotContext[MessageEvent]):
    return not incoming(ctx)


outgoing = create(outgoing_filter)
"""Filter outgoing messages. Messages that are sent by the user."""


async def community_filter(self, ctx: BotContext[Event], community_id: Optional[SCT[str]]):
    if community_id is None:
        return bool(ctx.event.community_id is not None)
    if isinstance(community_id, str):
        community_id = frozenset({community_id})
    else:
        community_id = frozenset(community_id)
    return bool(ctx.event.community_id in community_id)


community = create(community_filter)
"""Filter messages coming from a specific community."""


async def channel_filter(self, ctx: BotContext[Event], channel_id: Optional[SCT[str]]):
    if channel_id is None:
        return bool(ctx.event.channel_id is not None)
    if isinstance(channel_id, str):
        channel_id = frozenset({channel_id})
    else:
        channel_id = frozenset(channel_id)
    return bool(ctx.event.channel_id in channel_id)


channel = create(channel_filter)
"""Filter messages coming from a specific channel."""


async def group_filter(self, ctx: BotContext[Event], group_id: Optional[SCT[str]]):
    if group_id is None:
        return bool(ctx.event.group_id is not None)
    if isinstance(group_id, str):
        group_id = frozenset({group_id})
    else:
        group_id = frozenset(group_id)
    return bool(ctx.event.group_id in group_id)


group = create(group_filter)
"""Filter messages coming from a specific group."""


async def user_filter(self, ctx: BotContext[Event], user_id: Optional[SCT[str]]):
    if user_id is None:
        return bool(ctx.event.action_by_id is not None)
    if isinstance(user_id, str):
        user_id = frozenset({user_id})
    else:
        user_id = frozenset(user_id)
    return bool(ctx.event.action_by_id in user_id)


user = create(user_filter)
"""Filter actions coming from a specific user."""


def text(text: Optional[SCT[str]]):
    async def func(self, ctx: BotContext[MessageEvent]):
        text = self.text
        if text is None:
            return bool(ctx.event.message is not None and ctx.event.message.message is not None)
        if isinstance(text, str):
            text = frozenset({text})
        else:
            text = frozenset(text)

        if ctx.event.type == EventType.MESSAGE:
            value = ctx.event.message.message
        elif ctx.event.type == EventType.COMMAND:
            value = ctx.event.command
        elif ctx.event.type == EventType.CALLBACK_QUERY:
            value = ctx.event.callback_data

        for t in text:
            if t in value:
                return True
            else:
                try:
                    regexp = re.compile(t)
                    if regexp.search(value):
                        return True
                except re.error:
                    pass
        return False

    return create(func, name="TextFilter", text=text)


""" Filter messages by text. """


def regexp(regexp: Optional[SCT[str]]):
    async def func(self, ctx: BotContext[MessageEvent]):
        regexp = self.regexp
        if regexp is None:
            return bool(ctx.event.message is not None and ctx.event.message.message is not None)
        if isinstance(regexp, str):
            regexp = frozenset({regexp})
        else:
            regexp = frozenset(regexp)

        if ctx.event.type == EventType.MESSAGE:
            value = ctx.event.message.message
        elif ctx.event.type == EventType.COMMAND:
            value = ctx.event.command
        elif ctx.event.type == EventType.CALLBACK_QUERY:
            value = ctx.event.callback_data

        if value is None:
            return False

        for r in regexp:
            try:

                cr = re.compile(r)
                if cr.match(value):
                    return True
            except re.error:
                pass
        return False

    return create(func, name="RegexpFilter", regexp=regexp)


""" Filter messages by regexp. """



from setuptools import setup, find_packages

with open("requirements.txt", encoding="utf-8") as r:
    requires = [i.strip() for i in r]

try:
    import pypandoc

    long_description = pypandoc.convert_file("README.md", "rst")
except (IOError, ImportError):
    long_description = open("README.md").read()


setup(
    name="swibots",
    version="1.0.14",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/switchcollab/Switch-Bots-Python-Library",
    description="Switch bot api",
    author="switchadmin",
    author_email="support@switch.pe",
    license="LGPLv3",
    python_requires=">=3.10",
    install_requires=requires,
)



import swibots
from swibots.utils.types import JSONDict
from swibots.base import SwitchObject

from .base_typed_inline_query_result import BaseTypedInlineQueryResult
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup
from .types import InlineQueryResultType


class InlineQueryResultVideo(BaseTypedInlineQueryResult):
    def __init__(self,
                 id: str,
                 title: str,
                 video_url: str,
                 mime_type: str = None,
                 video_width: int = None,
                 video_height: int = None,
                 video_duration: int = None,
                 caption: str = None,
                 input_message: "InputMessageContent" = None,
                 description: str = None,
                 thumb_url: str = None,
                 thumb_width: int = None,
                 thumb_height: int = None,
                 reply_markup: "InlineMarkup" = None,
                 ):
        super().__init__(id, InlineQueryResultType.VIDEO, title, description, thumb_url, thumb_width, thumb_height,
                         input_message, reply_markup)
        self.video_url = video_url
        self.video_width = video_width
        self.video_height = video_height
        self.video_duration = video_duration
        self.caption = caption
        self.mime_type = mime_type

    def to_json(self) -> JSONDict:
        data = super().to_json()
        data.update({
            "videoUrl": self.video_url,
            "videoWidth": self.video_width,
            "videoHeight": self.video_height,
            "videoDuration": self.video_duration,
            "caption": self.caption,
            "mimeType": self.mime_type,
        })
        return data



class WsMessage():
    def __init__(self, raw):
        self.raw = raw
        self.headers = raw.headers
        self.body = raw.body
        self.command = raw.command
        self.destination = raw.headers.get('destination')
        self.subscription = raw.headers.get('subscription')
        self.message_id = raw.headers.get('message-id')
        self.ack = raw.headers.get('ack')
        self.nack = raw.headers.get('nack')



from typing import Generic, Optional, TypeVar
import swibots
from swibots.api.common.events.event import Event
from swibots.api.community.models.channel import Channel
from swibots.api.community.models.community import Community
from swibots.api.community.models.group import Group
from swibots.api.common.models.user import User
from swibots.types import EventType
from swibots.utils.types import JSONDict

T = TypeVar("T", bound="CommunityEvent")


class CommunityEvent(Event, Generic[T]):
    def __init__(
        self,
        app: "swibots.App" = None,
        type: Optional[EventType] = None,
        community_id: Optional[str] = None,
        community: Optional[Community] = None,
        group_id: Optional[str] = None,
        group: Optional[Group] = None,
        channel_id: Optional[str] = None,
        channel: Optional[Channel] = None,
        action_by_id: Optional[str] = None,
        action_by: Optional[User] = None,
        data: Optional[dict] = None,
        user_id: Optional[str] = None,
        user: Optional[User] = None,
    ):
        super().__init__(
            app=app,
            type=type,
            data=data,
            action_by=action_by,
            action_by_id=action_by_id,
            community=community,
            community_id=community_id,
            group=group,
            group_id=group_id,
            channel=channel,
            channel_id=channel_id,
        )
        self.user_id = user_id
        self.user = user

    def to_json(self) -> JSONDict:
        d = super().to_json()

        d.update(
            {
                "communityId": self.community_id,
                "community": self.community.to_json(),
                "groupId": self.group_id,
                "group": self.group.to_json(),
                "channelId": self.channel_id,
                "channel": self.channel.to_json(),
                "userId": self.user_id,
                "user": self.user.to_json(),
            }
        )
        return d

    def from_json(self, data: JSONDict) -> T:
        if data is not None:
            super().from_json(data)
            details = data.get("details") or {}
            self.user_id = data.get("userId")
            self.user = User.build_from_json(details.get("user"), self.app)
        return self



from typing import List
import swibots
from swibots.api.chat.models import Message


class GetCommunityMediaFiles:
    async def get_community_media_files(
        self: "swibots.ApiClient", community_id: str
    ) -> List[Message]:
        """Get community media files

        Parameters:
            community_id (``int``): The community id

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_messages`.
        """
        return await self.chat_service.messages.get_messages(community_id)



from .clear_conversation import ClearConversation
from .delete_message import DeleteMessage
from .delete_messages_from_user import DeleteMessagesFromUser
from .edit_message import EditMessage
from .flag_message import FlagMessage
from .forward_message import ForwardMessage
from .get_channel_chat_history import GetChannelChatHistory
from .get_group_chat_history import GetGroupChatHistory
from .get_community_media_files import GetCommunityMediaFiles
from .get_community_media_files_by_status import GetCommunityMediaFilesByStatus
from .get_flag_messages import GetFlagMessages
from .flag_message import FlagMessage
from .get_message import GetMessage
from .get_messages_between_users import GetMessagesBetweenUsers
from .get_messages import GetMessages
from .get_unread_messages_count import GetUnreadMessagesCount
from .get_user_media_files import GetUserMediaFiles
from .send_message import SendMessage
from .send_text import SendText
from .reply_message_text import ReplyMessageText
from .reply_message import ReplyMessage
from .edit_message_text import EditMessageText
from .answer_inline_query import AnswerInlineQuery
from .download_media import DownloadMedia


class ChatMethods(
    ClearConversation,
    DeleteMessage,
    DeleteMessagesFromUser,
    EditMessage,
    FlagMessage,
    ForwardMessage,
    GetChannelChatHistory,
    GetGroupChatHistory,
    GetCommunityMediaFiles,
    GetCommunityMediaFilesByStatus,
    GetFlagMessages,
    GetMessage,
    GetMessagesBetweenUsers,
    GetMessages,
    GetUnreadMessagesCount,
    GetUserMediaFiles,
    SendMessage,
    SendText,
    ReplyMessageText,
    ReplyMessage,
    EditMessageText,
    AnswerInlineQuery,
    DownloadMedia,
):
    pass



import enum


class InlineQueryResultType(enum.Enum):
    """Represents the type of a inline query result."""

    ARTICLE = "ARTICLE"
    PHOTO = "PHOTO"
    DOCUMENT = "DOCUMENT"
    VIDEO = "VIDEO"



from typing import Type, TypeVar
import swibots


class DeleteBotInfo:
    async def delete_bot_info(self: "swibots.ApiClient", bot_id: str) -> bool:
        """Delete bot info

        Parameters:
            bot_id (``str``): The bot id. Defaults to the current bot id.

        Returns:
            ``bool``: True if the bot info was deleted

        This functions does the same as :meth:`~switch.api.bot.controllers.BotController.delete_bot_info`.
        """
        return await self.bots_service.bots.delete_bot_info(bot_id=bot_id)



from typing import List
import swibots
from swibots.api.chat.models import Message


class GetUserMediaFiles:
    async def get_user_media_files(
        self: "swibots.ApiClient",
        user_id: int = None,
    ) -> List[Message]:
        """Get user media files


        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The user media files

        Raises:
            ``~switch.error.SwitchError``: If the user media files could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_user_media_files`.
        """
        return await self.chat_service.messages.get_user_media_files(user_id)



from typing import Optional
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject


class User(SwitchObject):
    def __init__(
        self,
        id: Optional[int] = None,
        name: Optional[str] = None,
        username: Optional[str] = None,
        image_url: Optional[str] = None,
        active: Optional[bool] = None,
        deleted: Optional[bool] = None,
        role_info: Optional[str] = None,
        admin: Optional[bool] = None,
        is_bot: Optional[bool] = None,
    ):
        self.id = id
        self.name = name
        self.username = username
        self.image_url = image_url
        self.active = active
        self.deleted = deleted
        self.role_info = role_info
        self.admin = admin
        self.is_bot = is_bot

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "name": self.name,
            "username": self.username,
            "imageUrl": self.image_url,
            "active": self.active,
            "deleted": self.deleted,
            "roleInfo": self.role_info,
            "admin": self.admin,
            "is_bot": self.is_bot,
        }

    def from_json(self, data: Optional[JSONDict] = None) -> "User":
        if data is not None:
            self.id = data.get("id")
            self.name = data.get("name")
            self.username = data.get("username")
            self.image_url = data.get("imageUrl")
            self.active = data.get("active")
            self.deleted = data.get("deleted")
            self.role_info = data.get("roleInfo")
            self.admin = data.get("admin")
            self.is_bot = data.get("is_bot")
        return self



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Generic, Optional, TypeVar
from swibots.bots.filters.filter import Filter
from swibots.utils.types import HandlerCallback

if TYPE_CHECKING:
    from swibots.bots.bot import Bot
    from switch.switch_client import SwitchApp


CtxType = TypeVar("CtxType")
ResType = TypeVar("ResType")


class BaseHandler(Generic[CtxType, ResType], ABC):
    def __init__(
        self,
        callback: HandlerCallback[CtxType, ResType],
        filter: Optional[Filter] = None,
        **kwargs,
    ):
        self.callback = callback
        self.filter = filter

    async def on_app_start(self, app: "SwitchApp"):
        pass

    async def on_app_stop(self, app: "SwitchApp"):
        pass

    async def should_handle(self, context: CtxType) -> bool:
        return False

    async def handle(
        self,
        context: CtxType,
    ) -> ResType:
        if await self.should_handle(context=context) and self.callback is not None:
            return await self.callback(context)



import swibots
from swibots.utils.types import JSONDict
from swibots.base import SwitchObject

from .base_typed_inline_query_result import BaseTypedInlineQueryResult
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup
from .types import InlineQueryResultType


class InlineQueryResultPhoto(BaseTypedInlineQueryResult):
    def __init__(self,
                 id: str,
                 title: str,
                 photo_url: str,
                 mime_type: str = None,
                 photo_width: int = None,
                 photo_height: int = None,
                 caption: str = None,
                 input_message: "InputMessageContent" = None,
                 description: str = None,
                 thumb_url: str = None,
                 thumb_width: int = None,
                 thumb_height: int = None,
                 reply_markup: "InlineMarkup" = None,
                 ):
        super().__init__(id, InlineQueryResultType.PHOTO, title, description, thumb_url, thumb_width, thumb_height,
                         input_message, reply_markup)
        self.photo_url = photo_url
        self.photo_width = photo_width
        self.photo_height = photo_height
        self.caption = caption
        self.mime_type = mime_type

    def to_json(self) -> JSONDict:
        data = super().to_json()
        data.update({
            "photoUrl": self.photo_url,
            "photoWidth": self.photo_width,
            "photoHeight": self.photo_height,
            "caption": self.caption,
            "mimeType": self.mime_type,
        })
        return data
